/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.PageTemplate");

dojo.require("dijit._Widget");
dojo.require("dijit._WidgetBase");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("jazz.ui.Menu");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.ui.internal.Drawer");
dojo.require("jazz.ui.MenuPopup");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.ui.internal.TruncatedTextNode");

dojo.requireLocalization("jazz.ui", "PageTemplateMessages");

(function() {

var messages;

/**
 * @name jazz.ui.PageTemplate.StatusMessage
 * @namespace <p>A set of constants for publishing events to the PageTemplate's status message.</p>
 */ 
var StatusMessage = {
	/** 
	 * Show a status message with the given message. 
	 * Ex. dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [message]);
	 * @constant
	 */
	SHOW: "jazz-ui-PageTemplate-StatusMessage-show",

	/** 
	 * Hide the status message.
	 * Ex. dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
	 * @constant
	 */
	HIDE: "jazz-ui-PageTemplate-StatusMessage-hide"
};

/**
 * Constructs a new PageTemplate, which provides the basic layout for a Jazz-based web UI.
 * The layout includes a banner, navbar, content area and footer, as well as various
 * hooks for icons, titles, home menu, mini dashboard, search providers and status
 * message.
 * 
 * See dijit._Container documentation, which provides API for managing children, for more
 * information about managing nodes in the content area of this widget.
 * 
 * All fields may be initialized in the widget constructor or  in a template, and may be 
 * changed after construction by using the attr function.
 * 
 * Note: Make sure that this.startup() is called once this widget is placed in the DOM. This is
 * important if you are instantiating this widget programmatically (this will happen automatically 
 * if instantiated from a template).
 * 
 * @name jazz.ui.PageTemplate
 * @class <p>A basic layout for Jazz-based web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 * @extends dijit._Container
 * @since 3.0
 */ 
dojo.declare("jazz.ui.PageTemplate", [dijit._Widget, dijit._Templated, dijit._Container], {

	/**
	 * An aboveBanner widget.  This goes above the title and spans the full width of the document. 
	 * This is often used for branding (ex. jazz.net navbar, jazz theming "aboveBanner" widget).
	 * @type DomNode|widget|String
	 * @default ""
	 */
	aboveBanner: "",

	/**
	 * A icon to appear in the banner, beside the bannerTitle. This should be a 24x24 icon.
	 * @type String(src)|object(src, className)
	 * @default ""
	 */
	bannerIcon:	"",
	
	/**
	 * A bannerRight widget. This goes in the area at the right side of the banner, opposite the
	 * banner icon and title. This widget typically contains drop down menus for Help, Admin and User.
	 * @type DomNode|widget|String
	 * @default ""
	 */
	bannerRight: "",
	
	/**
	 * A title to appear in the banner. This text should indicate context, such as current project.
	 * @type String
	 * @default ""
	 */	
	bannerTitle: "",
	
	/**
	 * A drawer widget. This is a hook for Mini Dashboard, or anything that subclasses 
	 * jazz.ui.internal.Drawer. The drawer slides out from either the right or left side of 
	 * the page.  May be a widget instance or a string reference to the widget (for lazy instantiation).
	 * @type widget|String
	 * @default ""
	 */	
	drawer: "",
	
	/**
	 * A footer widget. This goes below the content area and spans the full width of the page.
	 * @type DomNode|widget|String
	 * @default ""
	 */	
	footer: "",
	
	/**
	 * A link to the desired home page of your application. This link is navigated to when the user 
	 * clicks on the home button at the left of the banner. For most Jazz-based applications, this should 
	 * link to personal dashboards on the Jazz Team Server. 
	 * @type String
	 * @default "#"
	 */
	homeLink: "#",
	
	/**
	 * A home menu provider function. This function should return a menu widget that will open below
	 * the home button at the left side of the banner, and is called when the user clicks on
	 * home button's down arrow.
	 * @type Function
	 * @default null
	 */	
	homeMenuProvider: null,

	/**
	 * An info widget.  This appears above and to the right of the banner and should contain a single line of 
	 * text (similar to the title attribute).
	 * @type DomNode|widget|String
	 * @default ""
	 */
	info: "",

	/**
	 * A navbar widget. This goes between the banner and the content area and spans the full width 
	 * of the page. If a searchProvider is specified, the navbar may be a sibling to a float: right element,
	 * so plan accordingly.
	 * @type DomNode|widget|String
	 * @default ""
	 */	
	navbar: "",	

	/**
	 * A searchProvider function. This function should return a widget that will be place at the right side
	 * of the navbar, typically a jazz.app.internal.BannerSearch widget.
	 * @type Function
	 * @default null
	 */	
	searchProvider: null,
	
	/**
	 * Specify the scroll mode of the PageTemplate. 
	 * @type String - "window": Scrolls the at the browser window level.
	 * 				  "content": Scrolls the content area, leaving header and footer fixed.
	 * @default "window"
	 */
	scrollMode: "window",
	
	/**
	 * A title for your page. This appears above the banner in small text. This text should
	 * indicate the application and instance names (i.e. the name of your product).
	 * @type String|DomNode|widget
	 * @default ""
	 */	
	title: "",
	
	/**
	 * An onResize event. Connect to this to be notified when the content area is resized.
	 * @type Function
	 */
	onResize: function() {},

	/**
	 * A function to add widgets to the content area of the page.
	 * @param {widget} widget A dijit._Widget
	 * @param {String|number} insertIndex The position to place the node within the content area.
	 */
	addChild: function(widget, insertIndex){
		this.inherited(arguments);
		if (this.drawer) {
			if (this.getChildren().length == 1) {
				this.drawer.set("contentMatchNode", widget);
			} else {
				this.drawer.set("contentMatchNode", this._content);
			}
		}
	},
	/**
	 * Returns the dimensions (h, w) of the content area of the PageTemplate.
	 * @param includeContent Specifies whether to include the content in the computation of the height and width.  
	 * 						 If false, the content will be hidden before the size is tested. Default: false. 
	 * @returns Object {h, w}
	 */
	getContentBox: function (includeContent){
		var hideContent = !includeContent && this.get("scrollMode") == "window",
			style = this.containerNode.style;
	
		if (hideContent) style.display = "none";
	
		var box = dojo.contentBox(this._content);
		//these are always zero, so don't send them
		delete box.t;
		delete box.l;
		
		if (hideContent) style.display = "";	
	
		return box;
	},
	
	/**
	 * Show a status message, a little green message that floats above the page at the top/center.
	 * A "..." will automatically be appended to the end of the string if not already present unless
	 * requested to show the message verbatim.
	 * @param {String} message A message to display in the status message area.
	 * @param {boolean} verbatim Optional. Defaults to false. Whether to display the given message
	 * 							 verbatim or to append ellipses to the given message.
	 */	
	showStatusMessage: function(message, verbatim) {
		if (!dojo.isString(message)) {
			console.error("PageTemplate.showStatusMessage must be passed a string.");
			return;
		}
		// append "..." if not already present
		if (!verbatim && message.indexOf("...", message.length - 3) === -1) {
			message += "...";
		}
		dojo.place(document.createTextNode(message), this._statusMessage, "only");
		if (!this._statusVisible) {
			if (this._fadeOutAnim.status() == "playing") {
				this._fadeOutAnim.stop();
			}
			this._statusMessage.style.display = "block";
			this._fadeInAnim.play();	
			this._statusVisible = true;
		}
		this._onScroll();
	},
	
	/**
	 * Hide the status message.
	 * @noparams
	 */	
	hideStatusMessage: function() {
		if(this._statusVisible){
			if (this._fadeInAnim.status() == "playing") {
				this._fadeInAnim.stop();
			}
			this._fadeOutAnim.play();
			this._statusVisible = false;
		}
	},
	
	// Internals

	constructor: function() {
		this._startupQueue = [];
		if (!messages) messages = dojo.i18n.getLocalization("jazz.ui", "PageTemplateMessages");
		this._messages = messages;
	},

	buildRendering: function() {
		this.inherited(arguments);
		//hide until startup is called to reduce flicker and improve performance.
		if (!this.serverTemplated)
			this.domNode.style.display = "none";	
	},
	
	startup: function() {
		dojo.addClass(document.documentElement, "jazz-ui");
		this.domNode.style.display = "";
		while (this._startupQueue.length > 0) {
			var widget = this._startupQueue.pop();
			widget&&widget.startup&&widget.startup();
		}
		this._onResize();
		this._resizeHandler = jazz.util.ResizeNotifier.register({context: this, callback: this._onResize, force: true, priority: 2});
		this.inherited(arguments);
	},
	
	postCreate: function() {
		if (!this.isLeftToRight()) {
			var temp = this._leftAnchor;
			this._leftAnchor = this._rightAnchor;
			this._rightAnchor = temp;
		}
		dojo.addClass(this._leftAnchor.domNode,"left");
		dojo.addClass(this._rightAnchor.domNode,"right");
		if (this.productName) {
			dojo.deprecated("jazz.ui.PageTemplate#productName","-- Set jazz.ui.PageTemplate#title instead","RC2");
		}
		if (this.productIcon) {
			dojo.deprecated("jazz.ui.PageTemplate#productIcon","-- Set jazz.ui.PageTemplate#bannerIcon instead","RC2");
		}
		
		// hook up status message events
		dojo.subscribe(StatusMessage.SHOW, this, "showStatusMessage");
		dojo.subscribe(StatusMessage.HIDE, this, "hideStatusMessage");
		
		// create animations for fade-in and fade-out
		var args = {node: this._statusMessage, duration: 250};
		this._fadeInAnim = dojo.fadeIn(args);
		this._fadeOutAnim = dojo.fadeOut(args);
		
		this.connect(this._fadeOutAnim,"onEnd",function() {
			this._statusMessage.style.display = "none";
		});
		
		this.connect(window,"onscroll",this._onScroll);
	},

	templatePath: dojo.moduleUrl("jazz.ui", "templates/PageTemplate.html"),
	
	widgetsInTemplate: true,

	_skipToContent: function(e) {
		e.preventDefault();
		this._content.tabIndex = "-1";
		this._content.focus();
	},
	uninitialize: function() {
		jazz.util.ResizeNotifier.unregister(this._resizeHandler);
		if (this._homeHoverWidget) {
			this._homeHoverWidget.destroy();
			delete this._homeHoverWidget;
		}
		if (this._homeNavbarWidget) {
			this._homeNavbarWidget.destroy();
			delete this._homeNavbarWidget;
		}
	},
	_setScrollModeAttr: function(scrollMode) {
		this.scrollMode = (scrollMode == "content") ? "content" : "window";
		dojo.toggleClass(document.documentElement, "scrollContent", scrollMode == "content");
		this._created&&this._updateHeight();
	},
	//setters (do not call directly - use this.set)
	_setTitleAttr: function(title) {
		dojo.empty(this._title);
		if (title === null || title === "") {
			this.title = null;
			return;
		} else if (dojo.isString(title)) {
			this._title.appendChild(document.createTextNode(title));
			this.title = title;
		} else {
			this.title = this._instantiateAndPlace(title, this._title);
		}
	},
	_setBannerTitleAttr: function(title) {
		if (this._truncatedNode) {
			this._truncatedNode.set("value", title);	
		} else {
			var tn = this._truncatedNode = new jazz.ui.internal.TruncatedTextNode({value: title});
			tn.placeAt(this._bannerTitle);
			if (this._started) {
				tn.startup();
			} else {
				this._startupQueue.push(tn);
			}
		}
		if (this._homeNavbarWidget) {
			this._homeNavbarWidget.set("title", dojo.string.substitute(messages.homeMenuContext, [title]));
		}
		this.bannerTitle = title;
	},
	_setBannerIconAttr: function(args) {
		this._bannerIcon.style.display = args ? "" : "none";
		var src = !args||dojo.isString(args) ? args : args.src;
		if (src) {
			if (!this._bannerIconImg) {
				var vals = {src: src};
				if (args && args.alt) {
					vals.alt = args.alt;
				} else {
					vals.alt = "";
				}
				this._bannerIconImg = dojo.create("img", vals, this._bannerIcon, "only");
			} else {
				this._bannerIconImg.src = src;	
			}
		} else {
			dojo.empty(this._bannerIcon);
		}
		this.bannerIcon = dojo.isString(args) ? src : args;
		this._bannerIcon.className = (args&&dojo.isString(args.className) ? args.className + " " : "") + "banner-icon";
		jazz.util.ResizeNotifier.poke();
	},
	_setBannerRightAttr: function(bannerRight) {
		this.bannerRight = this._instantiateAndPlace(bannerRight, this._headerRight);
	},
	_setInfoAttr: function(info) {
		this.info = this._instantiateAndPlace(info, this._info);
	},
	_setNavbarAttr: function(navbar) {
		this.navbar = this._instantiateAndPlace(navbar, this._links);
	},
	_setAboveBannerAttr: function(aboveBanner) {
		this.aboveBanner = this._instantiateAndPlace(aboveBanner, this._aboveBannerNode);
	},
	_setSearchProviderAttr: function(searchProvider) {
		this._instantiateAndPlace(searchProvider, this._searchArea);
	},
	_setDrawerAttr: function(drawer) {
		if (this.drawer && this.drawer instanceof dijit._WidgetBase) {
			this.disconnect(this._pinChangeConnect);
			this.drawer.destroy();
		}
		if (!drawer) {
			dojo.empty(this._leftAnchor);
			dojo.empty(this._rightAnchor);
			this.drawer = null;
			return;
		}
		if (this.getChildren().length == 1) {
			var child = this.getChildren()[0];
			if (child && child.domNode) {
				var matchNode = child.domNode;
			}
		}
		if (typeof drawer == 'string') {
			drawer = new (dojo.getObject(drawer))({
				leftAnchor: this._leftAnchor,
				rightAnchor: this._rightAnchor,
				navbarNode: this._navbarWrapper
			});
		} else {
			drawer.set("leftAnchor", this._leftAnchor);
			drawer.set("rightAnchor", this._rightAnchor);
			drawer.set("navbarNode", this._navbarWrapper);
		}
		this.drawer = drawer;
		if (this._started) {
			drawer.startup && drawer.startup();
		} else {
			this._startupQueue.push(drawer);
		}
		this._pinChangeConnect = this.connect(drawer, "onPinChange", "_drawerPinChange");
	},
	_drawerPinChange: function (){
		jazz.util.ResizeNotifier.poke();
	},
	_setFooterAttr: function(footer) {
		this.footer = this._instantiateAndPlace(footer, this._footer);
	},
	_setHomeLinkAttr: function(href) {
		this.homeLink = href;
		if (this._homeMenuButton) {
			this._homeMenuButton.href=href;
		}
	},
	_setHomeMenuProviderAttr: function(func) {
		if (this._homeMenuConnects) {
			while (this._homeMenuConnects.length > 0) this.disconnect(this._homeMenuConnects.pop());
		}
		if (this._homeHoverWidget) {
			this._homeHoverWidget.destroy();
		}
		if (func) {
			var a11y = dojo.hasClass(document.body, "dijit_a11y");
			if (!this._homeMenuButton) {
				this._homeMenuButton = dojo.create("a", {title: messages.home, "class": "home-button j-mirror", href: this.homeLink}, this._homeMenu, "only");
				if (a11y) dojo.place(document.createTextNode(messages.home), dojo.create("span", {}, this._homeMenuButton));				
				this._homeMenuAnchor = dojo.create("a",{title: messages.homeMenu, href: "#"});
			}
			this._homeHoverWidget = new jazz.ui.MenuPopup({
				menuProvider: func,
				"class": "home-menu",
				around: this._homeMenuButton,
				cacheMenu: true
			}, this._homeMenuAnchor).placeAt(a11y ? this._homeMenu : this._homeMenuButton);
			this._homeMenuConnects = [
				this.connect(this._homeHoverWidget.domNode, "onmouseover",this._onCaretOver),
				this.connect(this._homeHoverWidget.domNode, "onmouseout",this._onCaretOut)
			];
			if (a11y) dojo.place(document.createTextNode(messages.homeMenu), dojo.create("span", {}, this._homeMenuAnchor));

			this._homeNavbarWidget = new jazz.ui.MenuPopup({
				menuProvider: func,
				cacheMenu: true,
				"class": "jazz-ui-PageTemplate-homeNavbar",
				title: dojo.string.substitute(messages.homeMenuContext, [this.get("bannerTitle")]),
				injectCaret: true
			}, dojo.create("a", {"href": "#", innerHTML: "<span class=\"home-menu-img j-mirror\"></span>"})).placeAt(this._links, "first");
		} else {
			dojo.empty(this._homeMenu);
		}
	},
	_onCaretOver: function(e){
		dojo.addClass(this._homeMenuButton,"over-caret");
	},
	_onCaretOut: function(e) {
		dojo.removeClass(this._homeMenuButton,"over-caret");
	},
	_instantiateAndPlace: function(widget, container) {
		if (!widget) {
			dojo.empty(container);
			return;
		}
		if (typeof widget == "function") {
			widget = widget();
		} else if (typeof widget == "string") {
			var obj = dojo.getObject(widget);
			if (!obj) {
				console.error("PageTemplate error: Could not instantiate \"" + widget + "\"");
				return "";
			}
			widget = new obj();
		} else if (widget.nodeType) {
			dojo.place(widget,container,"only");
			return widget;
		} 
		if (widget.domNode) {
			dojo.place(widget.domNode,container,"only");
			if (this._started) {
				widget.startup&&widget.startup();
			} else {
				this._startupQueue.push(widget);
			}
			return widget;
		}
		return "";
	},
	_onResize: function() {
		this._updateHeight();
		this.onResize(); //trigger onResize event
	},
	_updateHeight: function(){
		var height = dijit.getViewport().h - dojo.position(this._table).y - dojo.position(this._footerWrapper).h;
		if (height < 0) height = 0;
		this._onScroll();
		this._content.style[(dojo.isIE >= 8 && this.get("scrollMode") == "window") ? "minHeight" : "height"] = height + "px";
	},
	_fixed: false,
	_onScroll: function(e) {
		var scrollTop = document.documentElement.scrollTop || document.body.scrollTop,
			navbarWrapper = this._navbarWrapper,
			navPos = dojo.position(navbarWrapper, true),
			navbarVertPadding = 6,
			msg = this._statusMessage;
		if (this._statusVisible) {
			var viewport = dijit.getViewport();
			var width = viewport.w, position = dojo.position(msg);
			msg.style.right = width ? (width / 2 - position.w / 2) + "px" : "";
		}
		this._started&&this.drawer&&this.drawer.checkHeight();
		if (scrollTop > navPos.y - 2){
			if (!this._fixed) {
				if (navPos.h > navbarVertPadding) {
					navbarWrapper.style.height = navPos.h + "px";
					msg.style.top = navPos.h + "px";
					dojo.addClass(this._navbar, "nav-fixed");
					//if the home menu is open, hide it, and show it around the new navbar node instead
					if (this._homeHoverWidget && this._homeNavbarWidget && this._homeHoverWidget._menuOpen) {
						this._homeHoverWidget.close(false);
						this._homeNavbarWidget.open();
					}
				} else {
					msg.style.top = "0";
					if (this.drawer && this.drawer.get("navbarNode")) {
						this.drawer.set("navbarNode", null);
					}
				}
				msg.style.position = "fixed";
				
				this._fixed = true;
			}
		} else {
			if (this._fixed) {
				if (navPos.h > navbarVertPadding) {
					dojo.removeClass(this._navbar, "nav-fixed");
					navbarWrapper.style.height = "";
					//if the home menu is open, hide the sticky navbar one, and show it around the home button instead
					if (this._homeHoverWidget && this._homeNavbarWidget && this._homeNavbarWidget._menuOpen) {
						this._homeNavbarWidget.close(false);
						this._homeHoverWidget.open();
					}
				}
				msg.style.top = "";
				msg.style.position = "";
				this._fixed = false;
			}
			if (this.drawer && !this.drawer.get("navbarNode")) {
				this.drawer.set("navbarNode", this._navbarWrapper);
			}
		}
	}
});

jazz.ui.PageTemplate.StatusMessage = StatusMessage;

})();