/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.util.dom");

/**
 * @namespace
 * <p>
 * Provides implementations of commonly used W3C DOM Level 2 and 3 methods that are namespace-
 * aware across all supported browsers. Native browser support is used where available. Otherwise,
 * the logic is implemented in Javascript.
 * </p>
 * <p>
 * This allows developers to read XML documents using a common API on all browsers, without having
 * to consider browser incompatibilities or lack of support.
 * </p>
 * 
 * Limitations:
 * <ul>
 * <li>Creating new attributes or elements with new namespaces (not yet defined) is not
 *    supported.</li>
 * <li>Modifying namespace declarations is not supported.</li>
 * <li><code>DOMExceptions</code> are not thrown consistently on all browsers. You should
 *    not rely on them being thrown.</li>
 * </ul>
 * 
 * @see <a href="http://www.w3.org/TR/DOM-Level-3-Core/">http://www.w3.org/TR/DOM-Level-3-Core/</a>
 * @since 4.0
 */
jazz.util.dom = {
	
	/**
	 * Creates an element of the given qualified name and namespace URI.
	 * 
	 * @param {Document} document The <code>Document</code> to create the element for.
	 * @param {string} namespaceURI The namespace URI of the element to create.
	 * @param {string} qualifiedName The qualified name of the element type to instantiate.
	 * @return {Element} A new <code>Element</code> object with the given qualified name and namespace URI.
	 * @see <a href="http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-DocCrElNS">http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-DocCrElNS</a>
	 */
	createElementNS: function(/* Document */ document, /* string */ namespaceURI, /* string */ qualifiedName) {
		if (document.createElementNS) {
			return document.createElementNS(namespaceURI, qualifiedName);
		}
		// browser doesn't support createElementNS; just use the qualified name
		// assumption: browsers either support all *NS methods or none
		return document.createElement(qualifiedName);
	},
	
	/**
	 * Retrieves an attribute value by local name and namespace URI.
	 * 
	 * @param {Element} elem The <code>Element</code> whose attribute to retrieve.
	 * @param {string} namespaceURI The namespace URI of the attribute to retrieve.
	 * @param {string} localName The local name of the attribute to retrieve.
	 * @return {string} The attribute's value as a string, or the empty string if it doesn't exist.
	 * @see <a href="http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElGetAttrNS">http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElGetAttrNS</a>
	 */
	getAttributeNS: function(/* Element */ elem, /* string */ namespaceURI, /* string */ localName) {
		if (elem.getAttributeNS) {
			// DOM4 (supported by e.g. FF >= 13) says return null instead of empty string 
			// but here we support only DOM3, so return empty string
			return elem.getAttributeNS(namespaceURI, localName) || "";
		}
		// browser doesn't support getAttributeNS; do it in javascript
		var attr = getAttributeNodeNS(elem, namespaceURI, localName);
		return attr ? attr.value : "";
	},
	
	/**
	 * Retrieves an attribute node by local name and namespace URI.
	 * 
	 * @param {Element} elem The <code>Element</code> whose attribute to retrieve.
	 * @param {string} namespaceURI The namespace URI of the attribute to retrieve.
	 * @param {string} localName The local name of the attribute to retrieve.
	 * @return {Attr} The attribute node with the specified local name and namespace URI,
	 *    or <code>null</code> if there is no such attribute.
	 * @see <a href="http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElGetAtNodeNS">http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElGetAtNodeNS</a>
	 */
	getAttributeNodeNS: function(/* Element */ elem, /* string */ namespaceURI, /* string */ localName) {
		if (elem.getAttributeNodeNS) {
			return elem.getAttributeNodeNS(namespaceURI, localName);
		}
		// browser doesn't support getAttributeNodeNS; do it in javascript
		// xmlns attribute implicitly has the standard xmlns namespace
		if (namespaceURI && (localName !== XMLNS || namespaceURI !== NS_XMLNS)) {
			// look for localName matches
			var attrs = elem.attributes;
			var len = attrs.length;
			for (var i=0;i<len;++i) {
				var attr = attrs[i];
				if (attr.baseName === localName) {
					// use namespaceURI as an optimization - it's not always there, but
					// it if is, then use it
					if (attr.namespaceURI === namespaceURI) {
						return attr;
					}
					// verify that prefix exists and maps to namespaceURI
					var prefix = attr.prefix;
					if (prefix && lookupNamespaceURI(elem, prefix) === namespaceURI) {
						return attr;
					}
				}
			}
			return null;
		}
		return elem.getAttributeNode(localName);
	},
	
	/**
	 * Look up the namespace URI associated to the given prefix, starting from this node.
	 * <p>
	 * Note: W3C DOM Level 3 specification is unclear about the expected results for built-in
	 * prefixes <code>xml</code> and <code>xmlns</code>. This implementation will always return
	 * the built-in namespace values, declared or not.
	 * </p>
	 * Limitations:
	 * <ul>
	 * 	<li>Not supported for attribute nodes.</li>
	 * </ul>
	 * 
	 * @param {Node} node The <code>Node</code> from which to start looking.
	 * @param {string} prefix The prefix to look for. If this parameter is <code>null</code>,
	 *    the method will return the default namespace URI, if any.
	 * @return {string} Returns the associated namespace URI or <code>null</code> if none is found.
	 * @see <a href="http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-lookupNamespaceURI">http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-lookupNamespaceURI</a>
	 */
	lookupNamespaceURI: function(/* Node */ node, /* string */ prefix) {
		// browser implementations are inconsistent for built-in namespaces "xml" and "xmlns" -
		// FF resolves these but WebKit doesn't; always return them here, to be consistent.
		if (prefix === XML) {
			return NS_XML;
		}
		else if (prefix === XMLNS) {
			return NS_XMLNS;
		}
		else if (node.lookupNamespaceURI) {
			// FF returns "" if not found; we return null for consistency
			return node.lookupNamespaceURI(prefix) || null;
		}
		// browser doesn't support Node#lookupNamespaceURI; do it in javascript
		// http://www.w3.org/TR/2004/REC-DOM-Level-3-Core-20040407/namespaces-algorithms.html
		switch (node.nodeType) {
			case 1: // ELEMENT_NODE
				return _lookupNamespaceURI(node, prefix);
			case 9: // DOCUMENT_NODE
				return _lookupNamespaceURI(node.documentElement, prefix);
			case 6: // ENTITY_NODE
			case 12: // NOTATION_NODE
			case 10: // DOCUMENT_TYPE_NODE
			case 11: // DOCUMENT_FRAGMENT_NODE
				return null;
			default:
				var parentNode = node.parentNode;
				if (parentNode) {
					return _lookupNamespaceURI(parentNode, prefix);
				}
				return null;
		}
	},
	
	/**
	 * Look up the prefix associated to the given namespace URI, starting from this node.
	 * The default namespace declarations are ignored by this method.
	 * <p>
	 * Note: W3C DOM Level 3 specification is unclear about the expected results for built-in
	 * prefixes <code>xml</code> and <code>xmlns</code>. This implementation will always return
	 * the built-in namespace prefixes, declared or not.
	 * </p>
	 * Limitations:
	 * <ul>
	 * 	<li>Not supported for attribute nodes.</li>
	 * 	<li>Overridden and nested default namespaces are not supported.</li>
	 * </ul>
	 *   
	 * @param {Node} node The <code>Node</code> from which to start looking.
	 * @param {string} namespaceURI The namespace URI to look for.
	 * @return {string} Returns an associated namespace prefix if found or <code>null</code> if
	 *    none is found. If more than one prefix is associated to the namespace prefix, one of
	 *    the prefixes will be returned.
	 * @see <a href="http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-lookupNamespacePrefix">http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-lookupNamespacePrefix</a>
	 */
	lookupPrefix: function(/* Node */ node, /* string */ namespaceURI) {
		// resolve the built-in namespaces
		if (namespaceURI === NS_XML) {
			return XML;
		}
		else if (namespaceURI === NS_XMLNS) {
			return XMLNS;
		}
		else if (node.lookupPrefix) {
			return node.lookupPrefix(namespaceURI);
		}
		// browser doesn't support Node#lookupPrefix; do it in javascript
		// http://www.w3.org/TR/2004/REC-DOM-Level-3-Core-20040407/namespaces-algorithms.html
		if (!namespaceURI) {
			return null;
		}
		switch (node.nodeType) {
			case 1: // ELEMENT_NODE
				return _lookupPrefix(node, namespaceURI); 
			case 9: // DOCUMENT_NODE
				return _lookupPrefix(node.documentElement, namespaceURI); 
			case 6: // ENTITY_NODE
			case 12: // NOTATION_NODE
			case 11: // DOCUMENT_FRAGMENT_NODE
			case 10: // DOCUMENT_TYPE_NODE
				return null; // type is unknown
			default:
				var parentNode = node.parentNode;
				if (parentNode) {
					return _lookupPrefix(parentNode, namespaceURI);
				}
				return null;
		}
	},
	
	/**
	 * Removes an attribute by local name and namespace URI. If no attribute with this local
	 * name and namespace URI is found, this method has no effect.
	 * <p>
	 * Limitations:
	 * </p>
	 * <ul>
	 * 	<li>Removing a namespace declaration is not supported.</li>
	 * </ul>
	 * 
	 * @param {Element} elem The <code>Element</code> whose attribute to remove.
	 * @param {string} namespaceURI The namespace URI of the attribute to remove.
	 * @param {string} localName The local name of the attribute to remove.
	 * @see <a href="http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElRemAtNS">http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElRemAtNS</a>
	 */
	removeAttributeNS: function(/* Element */ elem, /* string */ namespaceURI, /* string */ localName) {
		if (elem.removeAttributeNS) {
			elem.removeAttributeNS(namespaceURI, localName);
		}
		else {
			// browser doesn't support removeAttributeNS; do it in javascript
			var attr = getAttributeNodeNS(elem, namespaceURI, localName);
			if (attr) {
				elem.removeAttributeNode(attr);
			}
		}
	},
	
	/**
	 * Adds a new attribute. If an attribute with the same local name and namespace URI is already
	 * present on the element, its prefix is changed to be the prefix part of the qualified name,
	 * and its value is changed to be the value parameter. This value is a simple string; it is not
	 * parsed as it is being set.
	 * <p>
	 * Limitations:
	 * </p>
	 * <ul>
	 * 	<li>Declaring a new namespace is not supported.</li>
	 * 	<li>Changing namespace prefixes is not supported).</li>
	 * 	<li>Setting namespace declaration attributes ("xmlns" or "xmlns:" prefix) is not supported.</li>
	 * </ul>
	 * 
	 * @param {Element} elem The <code>Element</code> whose attribute to create or alter.
	 * @param {string} namespaceURI The namespace URI of the attribute to create or alter.
	 * @param {string} qualifiedName The qualified name of the attribute to create or alter.
	 * @param {string} value The value to set in string form.
	 * @see <a href="http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElSetAttrNS">http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElSetAttrNS</a>
	 */
	setAttributeNS: function(/* Element */ elem, /* string */ namespaceURI, /* string */ qualifiedName, /* string */ value) {
		if (elem.setAttributeNS) {
			elem.setAttributeNS(namespaceURI, qualifiedName, value);
		}
		else {
			// browser doesn't support setAttributeNS; just set the qualified name
			elem.setAttribute(qualifiedName, value);
		}
	}
};

/*
 * Private
 */
var dom = jazz.util.dom;
var getAttributeNodeNS = dom.getAttributeNodeNS;
var lookupNamespaceURI = dom.lookupNamespaceURI;

var NS_XML = "http://www.w3.org/XML/1998/namespace";
var NS_XMLNS = "http://www.w3.org/2000/xmlns/";
var XML = "xml";
var XMLNS = "xmlns";

/*
 * Auxiliary function to implement Node#lookupNamespaceURI on some browsers.
 * See: http://www.w3.org/TR/2004/REC-DOM-Level-3-Core-20040407/namespaces-algorithms.html
 */
function _lookupNamespaceURI(/* Node */ node, /* string */ prefix) {
	do {
		var elemNamespaceURI = node.namespaceURI;
		if (elemNamespaceURI && node.prefix === prefix) {
			return elemNamespaceURI;
		}
		var attrs = node.attributes;
		var len = attrs.length;
		for (var i=0;i<len;++i) {
			var attr = attrs[i];
			if ((attr.prefix === XMLNS && attr.baseName === prefix) ||
					(prefix === null && attr.name === XMLNS)) {
				return attr.value || null;
			}
		}
		node = node.parentNode;
	} while (node && node.nodeType === 1)
	return null;
}

/*
 * Auxiliary function to implement Node#lookupPrefix on some browsers.
 * See: http://www.w3.org/TR/2004/REC-DOM-Level-3-Core-20040407/namespaces-algorithms.html
 */
function _lookupPrefix(/* Element */ elem, /* string */ namespaceURI) {
	do {
		var elemNamespaceURI = elem.namespaceURI;
		var prefix = elem.prefix;
		if (prefix && elemNamespaceURI && elemNamespaceURI === namespaceURI && lookupNamespaceURI(elem, prefix) === namespaceURI) {
			return prefix;
		}
		var attrs = elem.attributes;
		var len = attrs.length;
		for (var i=0;i<len;++i) {
			var attr = attrs[i];
			var baseName = attr.baseName;
			if (attr.prefix === XMLNS && attr.value === namespaceURI && lookupNamespaceURI(elem, baseName) === namespaceURI) {
				return baseName || null;
			}
		}
		elem = elem.parentNode;
	} while (elem && elem.nodeType === 1)
	return null;
}

})();
