/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.entryPoints");

dojo.require("jazz.app.internal.util.DiscoveredService");

(function() {
	
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var urls = {};

/**
 * Discovers Jazz UI entry points from the discovery service. Useful for discovering the url 
 * for Personal Dashboards, Lifecycle Project Admin, etc.
 * 
 * @param {String|Array} value A string or an array of strings containing UI entry point names.
 * @return {dojo.Deferred} The deferred callback will be passed the url or an array of urls (the 
 * 		same order you passed in).	i.e. if you passed in a String, it will pass the url string. 
 * 		If you passed in an array of strings, it will return you an array of url strings.  If any
 * 		entry point doesn't exist, null will be returned in its place.
 * 
 * Known UI entry point names:
 *   - PersonalDashboards
 *   - LifecycleProjectAdmin
 *   - Users
 *   - JazzTeamServerHome
 * 
 * Example usage:
 * 
 * var deferred = jazz.app.entryPoints.get(["PersonalDashboards", "LifecycleProjectAdmin"]);
 * deferred.addCallback(function(urls) {
 * 		var dashboardsUrl = urls[0];
 * 		var lpaUrl = urls[1];
 * 		//do stuff.
 * });
 * 
 */
jazz.app.entryPoints.get = function(value) {
	var deferred = new dojo.Deferred();
	var typesToFetch = [], valuesToFetch = [];
	if (dojo.isString(value)) {
		if (!urls[value]) {
			typesToFetch.push(NAMESPACES.juentry + value);
			valuesToFetch.push(value);
		}
	} else if (dojo.isArray(value)) {
		for (var i = 0; i < value.length; i++) {
			if (!urls[value[i]]) {
				valuesToFetch.push(value[i]);
				typesToFetch.push(NAMESPACES.juentry + value[i]);
			}
		}
	}
	if (typesToFetch.length > 0) {
		var discoveryDeferred = DiscoveredService.loadDiscoveryServices(typesToFetch, _getBinding(valuesToFetch));
		discoveryDeferred.addBoth(function(doc){
			if (dojo.isString(value)) {
				var url = doc[value];
				if (url) {
					deferred.callback(url);
					urls[value] = url;
				} else {
					deferred.callback(null);
				}
			} else if (dojo.isArray(value)) {
				var retArr = [];
				for (var j = 0; j < value.length; j++) {
					var url = doc[value[j]];
					if (url) {
						retArr.push(url);
						urls[value[j]] = url;
					} else {
						var url = urls[value[j]];
						retArr.push(url||null);
					}
				}
				deferred.callback(retArr);
			}
			return doc;
		});
	} else {
		if (dojo.isString(value)) {
			deferred.callback(urls[value]);
		} else if (dojo.isArray(value)) {
			var retArr = []; 
			for (var k = 0; k < value.length; k++) {
				retArr.push(urls[value[k]]);
			}
			deferred.callback(retArr);
		}
	}
	return deferred;
}

var NAMESPACES = {
	jd: "http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
	juentry: "http://jazz.net/ns/ui/entry#",
	rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
};

function _getBinding(/* string[] */ values) {
	var binding = {
		namespaces: NAMESPACES
	};
	var len = values.length;
	for (var i=0;i<len;++i) {
		var value = values[i];
		binding[value] = ["string", "juentry:" + value + "/jd:service/@rdf:resource", "1"];
	}
	return binding;
}

})();