/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.menu.MenuItem");

dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("jazz.util.moduleResource");

(function() {


/**
 * A more versatile version of dijit.MenuItem that:
 *   - For its icon, accepts either a className with background image or an image src (in order to support images that come from other servers).
 *   - For its onClick behavior, accepts either an onClick handler or an href (to support CTRL+Click and right click to open in new window).
 * @name jazz.ui.menu.MenuItem
 * @class <p>A versatile menu item for web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit.MenuItem
 * @extends dijit._Container
 */
dojo.declare("jazz.ui.menu.MenuItem", dijit.MenuItem, {
	
	/**
	 * URL of a 16x16 pixel image to use as the menu item's icon.  The URL may be absolute, relative or 
	 * module-relative. The value is passed through jazz.util.moduleResource, so, for 
	 * example, jazz.ui/graphics/myimage.png will resolve properly.
	 * @type String
	 * @default ""
	 */
	icon: "",
	
	/**
	 * Class that specifies a background image to use as the MenuItem's icon.
	 * @type String
	 * @default ""
	 */
	iconClass: "",
	
	/**
	 * URL to follow when the user clicks the menu item.
	 * @type String
	 * @default ""
	 */
	href: "",

	/**
	 * Specify whether this menu item is selected or not. If true, shows a black dot.
	 * @type boolean
	 * @default false
	 */
	selected: false,
	
	// internals
	templateString: null, // override base class template
	templatePath: dojo.moduleUrl("jazz.ui", "menu/templates/MenuItem.html"),
	
	postCreate: function() {
		this.inherited(arguments);
		this.focusNode.style.cursor="pointer";
		if (this.href) {
			var a = dojo.create("a", {"class": "itemHref", tabIndex: "-1", href: this.href});
			this.connect(a, "onclick", this._onClick);
			dojo.place(this._wrapper, a);
			dojo.place(a, this._td);
		}
	},
	_setIconAttr: function(icon) {
		icon = jazz.util.moduleResource(icon);
		this.icon = icon;
		this.iconNode.src = icon;
	},
	_setSelectedAttr: function(selected) {
		if (selected) {
			if (!this._bullet) this._bullet = dojo.create("span", {"class": "bullet"}, this._td, "first");
		} else if (this._bullet){
			this._td.removeChild(this._bullet);
			delete this._bullet;
		}
		dijit.setWaiState(this.domNode, "checked", selected);
		dojo.toggleClass(this.domNode, "selected", selected);
		this.selected = selected;
	},
	_onClick: function(evt) {
		if (this.disabled) {
			dojo.stopEvent(evt);
			return;
		}
		if (this.href) {
			if (evt.altKey || evt.ctrlKey || evt.shiftKey || evt.metaKey) {
				if (dojo.isFF) {
					// FF workaround for https://bugzilla.mozilla.org/show_bug.cgi?id=748740
					// Click event must propagate to the window to support ctrl-click
					try {
						// hack: prevent anything from canceling this event
						evt.stopPropagation = function(){};
						evt.preventDefault = function(){};
					}
					catch (e) {}
				}
				// prevent parent from stopping event, use default event handling
				evt.stopPropagation();
				return;
			}
			
			if (evt.button === dojo.mouseButtons.MIDDLE) {
				// See https://bugs.dojotoolkit.org/ticket/17344
				// We don't want anything (i.e. the menu) to cancel this event
				try {
					evt.stopPropagation = function(){};
					evt.preventDefault = function(){};
				}
				catch (e) {}
			} else {
				// normal clicks are handled by shouldVetoNavigation (ajax transitions) or default link event handler
				// (page transition). Prevent parent from stopping event
				evt.stopPropagation();
				var PlatformUI = dojo.getObject("net.jazz.ajax.ui.PlatformUI"); // optional dependency
				if (PlatformUI) {
					// if we have a workbench, check navigate away. this also
					// handles the ajax transition
					var wb = PlatformUI.getWorkbench();
					if (wb) {
						if (!wb.shouldVetoNavigation(this.href)) {
							location.href = this.href;
						}
						else {
							dojo.stopEvent(evt);
						}
					}
				}
			}
			// allow the menu to close for responsiveness
			var parent = this.getParent();
			setTimeout(dojo.hitch(parent, parent.onExecute), 0);
		}
		else if (this.onClick) {
			this.onClick(evt);
		}
	},
	_setStateClass: function() {
		if (!this.domNode) return;
		// prevent selected from being used by _CssStateMixin
		var sel = this.selected;
		this.selected = false;
		this.inherited(arguments);
		this.selected = sel;
	}
});
})();