/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportParamFilteredList");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.reports.web.ui.internal.ReportParam");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");
dojo.require("com.ibm.team.reports.web.ui.internal.FilteredSelectionDialog");
dojo.require("com.ibm.team.reports.web.ui.internal.SelectionDialog");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	
	var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;
	
	/**
	 * A report parameter that allows the user to select a value from a list, or type in a value directly. The list can
	 * optionally be filtered using a filter string entered by the user.
	 */
    dojo.declare("com.ibm.team.reports.web.ui.internal.ReportParamFilteredList", [com.ibm.team.reports.web.ui.internal.ReportParam], {
    	
    	/**
    	 * Template elements:
    	 * displayLabel: a label for showing the display name of the value (if available)
    	 */
        templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportParamFilteredList.html"),
        
        _dialog: null,
        _labels: null,
        _values: null,
        _selectedValue: null,
        _showFilter: false,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			
			this._showFilter = (this.parameterDTO.settings & this.statics.SHOW_FILTER) == this.statics.SHOW_FILTER;
			var hasDependencies = (this.parameterDTO.settings & this.statics.HAS_DEPENDENCIES) == this.statics.HAS_DEPENDENCIES;
				
			this.isDeferred = this._showFilter || hasDependencies;
		},
		
		postCreate: function() {
			this.connect(this.browseForParameter, "onsubmit", this._browseForParameterHandler);
			this.connect(this.editBox,"onchange",this._handleManualValueChange);
			
			this.displayLabel.appendChild(dojo.doc.createTextNode(this.messages.noLabel));
		},
		
		/**
		 * Overridden from superclass.  This type of parameter can be initialized multiple times.
		 */
		init: function(reportParameterDTO) {
			if (reportParameterDTO) {
				this.appendDefaultValues(reportParameterDTO, true);
				this.setDefaultValues();
			}
		},
		
		/**
		 * Overridden from superclass.
		 */
		setDefaultValues: function() {
			if (this.parameterDTO.defaultValue) {
				this._setValue({value:StringUtils.stripQuotes(this.parameterDTO.defaultValue)});
			}
		},
		
		/**
		 * Overridden from superclass.
		 */
		getSelectedValues: function() {
			var value = this.editBox.value;
			//an empty string should not count as a selected value
			return [value.length > 0 ? value : null];
		},
		
		/**
		 * Overriden from superclass.
		 */
		setValues: function(values) {
			this._setValue({value:StringUtils.stripQuotes(values)});			
		},
		
		/**
		 * Overriden from superclass.
		 */
		resetValues: function() {
			this._setValue({value: ""});
			this.setDefaultValues();
		},
		
		/**
		 * Overridden from superclass.
		 */
		appendDefaultValues: function(reportParameterDTO) {
			this._values = reportParameterDTO.values ? reportParameterDTO.values : [];
    		this._labels = reportParameterDTO.labels ? reportParameterDTO.labels : [];
    		
    		if (this._dialog) {
    			this._dialog.populateSelectionList(this._values, this._labels);
    		}
		},
		
		/**
		 * Handles the user manually inputting the parameter value.
		 */
		_handleManualValueChange: function(e) {
			//for a manually entered value, we don't know the display name, so remove it
			this.displayLabel.innerHTML = "";
			this.displayLabel.appendChild(dojo.doc.createTextNode(this.messages.noLabel));
		},
		
		_browseForParameterHandler: function(e) {
			e.preventDefault(); //since the button is defined in a form, we don't want to trigger form submit
			var params= {
					onOk: dojo.hitch(this, function(result) {
						delete this._dialog;
						this._setValue(result);
						this.onDirty();
					})					
				};
			
			if (this.isDeferred) {
				if (this._showFilter) {
					params.setFilter = dojo.hitch(this, function(filterString) {
						this.getValuesDeferred({filter:filterString});
					});
					this._dialog = new com.ibm.team.reports.web.ui.internal.FilteredSelectionDialog(params);
				} else {
					this.getValuesDeferred();
					this._dialog = new com.ibm.team.reports.web.ui.internal.SelectionDialog(params);
				}
			} else {
				params.values = this._values;
				params.labels = this._labels;
				this._dialog = new com.ibm.team.reports.web.ui.internal.SelectionDialog(params);
			}
		},

		remoteError: function(error) {
			alert(error.message);
		},
		
		/**
		 * Sets the current value of this param.
		 * 
		 * result: obj with fields value (required), label (optional)
		 */
		_setValue: function(result) {
			this.editBox.value = result.value;
			
			this.displayLabel.innerHTML = "";
			if (result.label) {
				this.displayLabel.appendChild(dojo.doc.createTextNode(result.label));
			} else {
				this.displayLabel.appendChild(dojo.doc.createTextNode(this.messages.noLabel));
			}
		},
		
	});
})();
