/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCustomEditorManager"); //$NON-NLS-1$
	dojo.require("jazz.core.RegistryFactory"); //$NON-NLS-1$
	dojo.require("jazz.core.loader"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	
	var EXTENSION_POINT_ID = "com.ibm.team.process.web.operationBehaviorEditors"; //$NON-NLS-1$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var extensions = null;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCustomEditorManager", null, { //$NON-NLS-1$
		
		constructor: function(args) {
			this._processAreaEditor = args.processAreaEditor;
			this._extensionStore = args.extensionStore;
			this._configurationStore = args.configurationStore;
			this._configurationDataModel = args.configurationDataModel;
			// roles store has to be manually kept up to date as of now...
			this.rolesStore = args.rolesStore;
			if (extensions == null) {
				extensions = [];
				var extensionPoint = jazz.core.RegistryFactory.getRegistry().getExtensionPoint(EXTENSION_POINT_ID);
				if (extensionPoint) {
					dojo.forEach(extensionPoint.getExtensions(), function(extension) {
						dojo.forEach(extension.getConfigurationElements(), function(element) {
							extensions.push(element);
						});
					});
				}
			}
			this._recentlyChanged = {};
		},
		
		/**
		 * Determines whether the given operation behavior item represents a
		 * precondition/follow-up action which requires specific custom
		 * configuration and for which no editor has already been created. If an
		 * editor has already been created for a precondition/ follow-up of this
		 * type, but not for this specific item, then a new editor is still
		 * needed.
		 * 
		 * @param item {dojo.data.Item} operation behavior item
		 * @returns {Boolean} true if a new editor must be instantiated for this
		 *          item, false otherwise
		 */
		needsNewEditor: function(item) {
			var itemId = this._configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
			var operationBehaviorId = itemId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR).pop();
			for (var i = 0; i < extensions.length; i++) {
				if (operationBehaviorId === extensions[i].getAttribute("operationBehaviorId")) { //$NON-NLS-1$
					return !this._getEditor(item);
				}
			}
			return false;
		},
		
		/**
		 * Finds the editor widget id of the given operation behavior item from
		 * the extensions of com.ibm.team.process.web.operationBehaviorEditors.
		 * 
		 * @param item {dojo.data.Item} operation behavior item
		 * 
		 * @returns {String} the widget id of the given operation behavior item
		 */
		getWidgetId: function(item) {
			if (item) {
				var itemId = this._configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
				var operationBehaviorId = itemId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR).pop();
				for (var i = 0; i < extensions.length; i++) {
					if (operationBehaviorId === extensions[i].getAttribute("operationBehaviorId")) { //$NON-NLS-1$
						return extensions[i].getAttribute("widget"); //$NON-NLS-1$
					}
				}
			}
		},
		
		/**
		 * Creates an editor for the given operation behavior item, if the
		 * precondition/follow-up action requires specific custom configuration.
		 * If such an editor has already been created, then just attaches it to
		 * the given parent node. If no editor is required (there is no
		 * extension point for this operation behavior id), does nothing.
		 * 
		 * @param item {dojo.data.Item} operation behavior item
		 * @param parentDomNode {HTMLElement} node in the HTML document to
		 * 	 		  append editor to
		 * @param widgetId {String} the editor widget id of this operation behavior item
		 * @returns {Widget} an instance of
		 *          com.ibm.team.process.web.ui.AbstractOperationBehaviorCustomEditor,
		 *          or null if this operation behavior does not require a
		 *          specific custom configuration editor
		 */
		createEditor: function(item, parentDomNode, widgetId) {
			var existing = this._getEditor(item);
			if (existing) {
				// already instantiated
				existing.aboutToShowEditor();
				parentDomNode.appendChild(existing.domNode);
				return existing;
			}
			
			if (widgetId) {
				// instantiate editor
				var itemId = this._configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
				var operationBehaviorId = itemId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR).pop();
				var widgetConstructor = eval(widgetId);
				var itemIdentity = this._configurationStore.getIdentity(item);
				try {
					var editor = new widgetConstructor({
						changesPendingCallback: dojo.hitch(this, function() {
							this._changesPendingCallback(item);
						}),
						processInfo: this._getProcessInfo()
					});
					this.editors[itemIdentity] = editor;
					// initialize editor
					var content = this._configurationStore.getValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTENT);
					if (content === null) {
						// this is a newly added precondition/follow-up OR
						// it may not be new, but this editor works directly
						// with XML and this behavior currently has no inner XML
						var extension = this._extensionStore.fetchItemByIdentity(itemId);
						if (this._extensionStore.getValue(extension, ProcessConstants.ExtensionDTO.HAS_XML_CONVERTER) !== true) {
							// this editor works directly with XML, create a
							// dummy XML node
							var type = itemId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR)[2];
							type = type === ProcessConstants.PRECONDITIONS ? ProcessConstants.PRECONDITION : ProcessConstants.FOLLOWUP_ACTION;
							content = "<" + type + "></" + type + ">"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						}
					}
					editor.initialize(content, operationBehaviorId);
					// add editor to document
					editor.aboutToShowEditor();
					parentDomNode.appendChild(editor.domNode);
					editor.startup();
				} catch (error) {
					if (this.editors[itemIdentity]) {
						this.editors[itemIdentity].destroyRecursive();
						delete this.editors[itemIdentity];
					}
					error.widgetId = widgetId;
					throw error;
				}
				return editor;
			}
			// no editor required (no matching extension)
			return null;
		},
		
		/**
		 * If an editor has been created for the given operation behavior item,
		 * then returns the string content of that behavior's configuration in
		 * the process specification (in XML format, or some other format if a
		 * converter is register for that behavior).
		 * 
		 * @param item {dojo.data.Item} operation behavior item
		 * @returns {String} configuration content for this operation behavior,
		 *          or null if no editor for this id
		 */
		getContent: function(item) {
			var content = null;
			var editor = this._getEditor(item);
			if (editor) {
				try {
					content = editor.getContent();
				} catch (error) {
					//There was a runtime exception executing the getContent()
					//function of a custom editor. This is most likely a
					//programmer error since the custom editor did not raise a
					//validation problem. Add the widget's identifier to the
					//error so that higher level code can display an appropriate
					//message
					error.customWidgetId = editor.declaredClass;
					throw error;
				}
			}
			return content;
		},
		
		/**
		 * Saves the configuration content of any recently updated custom
		 * editors to the configuration store.
		 */
		saveContent: function() {
			for (var id in this._recentlyChanged) {
				var item = this._recentlyChanged[id];
				var content = this.getContent(item);
				this._configurationStore.setValue(item, ProcessConstants.BehaviorConfigurationDTO.CONTENT, content);
			}
			this._recentlyChanged = {};
		},
		
		_getEditor: function(item) {
			if (!this.editors) {
				this.editors = {};
			}
			return this.editors[this._configurationStore.getIdentity(item)];
		},
		
		changesPending: function() {
			for (var id in this.editors || {}) {
				if (this.editors[id].changesPending()) {
					return true;
				}
			}
			return false;
		},
		
		canSave: function() {
			for (var id in this.editors || {}) {
				if (!this.editors[id].canSave()) {
					return false;
				}
			}
			return true;
		},
		
		/**
		 * Returns the most recent error object (if any) from this manager's
		 * editors.
		 * 
		 * @return {Object} error object with "id" and "error", or null
		 */
		validate: function() {
			if (this._errors && this._errors.length > 0) {
				return this._errors[0];
			}
			return null;
		},
		
		/**
		 * Checks with the custom editor for the given item to see if it is
		 * returning an error message. Maintains a stack of errors, with the
		 * most recent at the top, so that validate() can always return the most
		 * recent error without having to validate every editor. This method is
		 * only called when the given item's editor indicates that its content
		 * has changed, and then only validates said editor.
		 */
		_validate: function(item) {
			if (!this._errors) {
				this._errors = [];
			}
			var editor = this._getEditor(item);
			// remove the previous error for this editor from the stack
			var id = this._configurationStore.getIdentity(item);
			for (var i = 0; i < this._errors.length; i++) {
				if (this._errors[i].id === id) {
					this._errors.splice(i, 1);
					break;
				}
			}
			// if there is still an error, put it on top of the stack
			var error = editor.validate();
			if (error !== undefined && error !== null && dojo.string.trim(error).length !== 0) {
				this._errors.splice(0, 0, {
					id: id,
					error: error
				});
			}
		},
		
		_changesPendingCallback: function(item) {
			this._validate(item);
			this._processAreaEditor.setDirty(this._processAreaEditor.changesPending());
			this._recentlyChanged[this._configurationStore.getIdentity(item)] = item;
		},
		
		_getProcessInfo: function() {
			return new com.ibm.team.process.web.ui.operationbehavior.ProcessInfo({
				rolesStore: this.rolesStore,
				projectAreaItemId: this._processAreaEditor.projectAreaItemId,
				configurationDataModel: this._configurationDataModel
			});
		},
		
		/**
		 * Returns the value of the attribute "uaKey" in the editor extension. 
		 * 
		 * @param item {dojo.data.Item} operation behavior item
		 * @returns {String} the value of the attribute "uaKey" in the editor extension
		 */
		getUaKey: function(item) {
			var itemId = this._configurationStore.getValue(item, ProcessConstants.ExtensionDTO.ID);
			var operationBehaviorId = itemId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR).pop();
			for (var i = 0; i < extensions.length; i++) {
				if (operationBehaviorId === extensions[i].getAttribute("operationBehaviorId")) { //$NON-NLS-1$
					return extensions[i].getAttribute("uaKey"); //$NON-NLS-1$
				}
			}
		},
		
		destroyEditors: function() {
			this._recentlyChanged = {};
			delete this._errors;
			if (this.editors) {
				for (var id in this.editors) {
					if (this.editors[id]) {
						this.editors[id].destroyRecursive();
					}
				}
				delete this.editors;
			}
		}
		
	});
})();
