/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy"); //$NON-NLS-1$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	/**
	 * Return whether project operations should be filtered from view for the
	 * combination of editor context and current time period.
	 */
	var shouldFilterOutProjectOperations = function(editorContext, scopeStore, scopeItem) {
		return ProcessConstants.EditorContext.PROJECT === editorContext && scopeStore.getValue(scopeItem, "contextId") !== ""; //$NON-NLS-1$ //$NON-NLS-2$
	};
	
	/**
	 * Subclass of dijit.tree.ForestStoreModel that provides permission
	 * extensions to views.
	 * 
	 * Content is filtered based on the time period for which permission are
	 * shown, which may be updated via function setTimePeriod(). In iteration
	 * types, timelines, and iterations, only team operations are returned.
	 * 
	 * Functions to access properties of permission extensions: their
	 * description, identifier, and parent operation identifier are available.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel", [dijit.tree.ForestStoreModel], { //$NON-NLS-1$
		
		/**
		 * The following arguments are required:
		 * 
		 * store						dojo.data store of permission extensions,
		 * 								typically computed via the workingCopy's
		 * 								function loadPermissionExtensions()
		 * processAreaStore				Store of artifacts related to project or
		 * 								team, used for time scopes such as
		 * 								timelines and iteration types
		 * workingCopy					Shared instance of PermissionsWorkingCopy
		 * editorContext				Value in ProcessConstants.EditorContext
		 *                              for the process area editor
		 * timePeriod					Item in processAreaStore that represents
		 * 								the time period for which permissions
		 * 								are displayed. The time period determines
		 * 								whether project operations are filtered
		 * 								out 
		 */
		constructor: function(args) {
			this.childrenAttrs = [ProcessConstants.ExtensionDTO.ITEMS];
			
			this._filterOutProjectOperations = shouldFilterOutProjectOperations(args.editorContext, args.processAreaStore, args.timePeriod);
			delete args.timePeriod;
			
			this._filterText = ""; //$NON-NLS-1$
		},
		
		/**
		 * Get the action or operation id of a permission extension item.
		 */
		getActionId: function(/*Dojo.data item*/ item) {
			return this.store.getValue(item, ProcessConstants.ExtensionDTO.ID);
		},
		
		/**
		 * Get the id of an action's parent operation.
		 */
		getParentOperationId: function(actionId) {
			return this.workingCopy.getParentOperationId(actionId);
		},
		
		/**
		 * Get the description of a permission extension item.
		 */
		getDescription: function(/*Dojo.data item*/ item) {
			return this.store.getValue(item, ProcessConstants.ExtensionDTO.DESCRIPTION);
		},
		
		/**
		 * Should be called when permissions for a different time period are
		 * to be shown. The time period determines whether project operations
		 * are filtered out. If the filtering changes, onChildrenChange() will
		 * be called on appropriate parent items, which clients can listen to.
		 * 
		 * A boolean is returned that indicates whether the filtering of items
		 * was changed as a result of the scope changing.
		 */
		setTimePeriod: function(scopeItem) {
			var filter = shouldFilterOutProjectOperations(this.editorContext, this.processAreaStore, scopeItem);
			var filterChanged = false;
			if (this._filterOutProjectOperations !== filter) {
				this._filterOutProjectOperations = filter;
				this._refreshModel();
				filterChanged = true;
			}
			return filterChanged;
		},
		
		/**
		 * Called when operations/actions are to be filtered by text. If the
		 * filtering changes, onChildrenChange() will be called on appropriate
		 * parent items.
		 * 
		 * @param	filterText	Text to filter by
		 * @param	forceUpdate	If true, matching items will be re-computed
		 * 						even when the model is already filtering on
		 * 						given text. Useful if the model could have
		 * 						been changed to filter by a different criteria
		 * 						in between invocations of setFilterText()
		 */
		setFilterText: function(filterText, forceUpdate) {
			filterText = dojo.string.trim(filterText).toLowerCase();
			if (this._filterText !== filterText || forceUpdate === true) {
				this._filterText = filterText;
				this._computeTextFilter();
				this._refreshModel();
			}
		},
		
		/**
		 * Returns an array of items whose label matches the text filter
		 * previously passed to setFilterText().
		 * 
		 * In cases where an operation/action and one or more children match
		 * the filter, only the highest item in the hierarchy is returned.
		 *
		 * For the folowing tree of operations and actions and filter text "abc":
		 * 
		 * Category x
		 *     Operation abc
		 *         Action abc 1
		 *         Action def
		 *     Operation def
		 *         Action abc 2
		 * 
		 * the items for 'Operation abc' and 'Action abc 2' are returned.
		 */
		getTextFilterMatches: function() {
			return this._itemsMatchingText;
		},
		
		/**
		 * Overridden to account for filtered out items if necessary.
		 */
		mayHaveChildren: function(item) {
			if (this._filterOutProjectOperations) {
				return dojo.some(this.store.getValues(item, ProcessConstants.ExtensionDTO.ITEMS), function(child) {
					return !this._isProjectOnlyOperationOrCategory(child) && this._itemMatchesTextFilter(child);
				}, this);
			} else {
				if (this._matchesTextFilter === undefined) {
					return this.inherited(arguments);
				} else {
					return dojo.some(this.store.getValues(item, ProcessConstants.ExtensionDTO.ITEMS), function(child) {
						return this._itemMatchesTextFilter(child);
					}, this);
				}
			}
		},
		
		/**
		 * Overridden to filter if necessary
		 */
		getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError) {
			if (parentItem === this.root || this.store.isItem(parentItem)) {
				// filter the potential child items
				var oldCallback = callback;
				callback = dojo.hitch(this, function(items) {
					items = this._filterItems(items);
					oldCallback(items);
				});
			}
			
			this.inherited(arguments);
		},
		
		/**
		 * Return children without regard to the text filter that was specified.
		 * Other filters, such as to filter out project operations when a time
		 * period is selected, will still apply.
		 */
		getChildrenIgnoreTextFilter: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback) {
			callback(this._filterItems(this.store.getValues(parentItem, ProcessConstants.ExtensionDTO.ITEMS), true));
		},
		
		/**
		 * Invokes onChildrenChange() on every item in the model, causing the
		 * view to completely refresh its contents.
		 */
		_refreshModel: function() {
			var items = [this.root];
			while (items.length > 0) {
				var item = items.shift();
				this.getChildren(item, dojo.hitch(this, function(newChildren) {
					this.onChildrenChange(item, newChildren);
					items = items.concat(newChildren);
				}));
			}
		},
		
		_filterItems: function(items, ignoreTextFilter) {
			if (this._filterOutProjectOperations) {
				var self = this;
				items = dojo.filter(items, function(item) {
					return !self._isProjectOnlyOperationOrCategory(item) && (ignoreTextFilter === true || self._itemMatchesTextFilter(item));
				});
			} else if (!ignoreTextFilter && this._matchesTextFilter !== undefined) {
				var self = this;
				items = dojo.filter(items, function(item) {
					return self._itemMatchesTextFilter(item);
				});
			}
			return items;
		},
		
		_computeTextFilter: function() {
			delete this._matchesTextFilter;
			this._itemsMatchingText = [];
			if (this._filterText.length !== 0) {
				var matchesTextFilter = {};
				var terms = this._filterText.split(" "); //$NON-NLS-1$
				var self = this;
				
				var processItem = function(item, parentMatchesTextFilter) {
					var itemMatchesTextFilter = parentMatchesTextFilter;
					if (!itemMatchesTextFilter) {
						var label = self.getLabel(item).toLowerCase();
						for (var i = 0; i < terms.length && (i === 0 || itemMatchesTextFilter); i++) {
							itemMatchesTextFilter = label.indexOf(terms[i]) !== -1;
						}
						
						if (itemMatchesTextFilter) {
							self._itemsMatchingText.push(item);
						}
					}
					var childMatchesTextFilter = false;
					self.getChildren(item, function(childItems) {
						dojo.forEach(childItems, function(childItem) {
							childMatchesTextFilter = processItem(childItem, itemMatchesTextFilter) || childMatchesTextFilter;
						});
					});
					if (!itemMatchesTextFilter) {
						itemMatchesTextFilter = childMatchesTextFilter;
					}
					if (itemMatchesTextFilter) {
						matchesTextFilter[self.getActionId(item)] = true;
					}
					return itemMatchesTextFilter;
				};
				
				this.getChildren(this.root, function(categoryItems) {
					dojo.forEach(categoryItems, function(categoryItem) {
						processItem(categoryItem, false);
					});
				});
				
				this._matchesTextFilter = matchesTextFilter;
			}
		},
		
		/*
		 * Given an item from the extensions store, determine whether it is a
		 * project operation or category with no team operations and therefore
		 * should not display in inherited scopes.
		 */
		_isProjectOnlyOperationOrCategory: function(operationItem) {
			return this.workingCopy.isProjectOperation(operationItem) || this.workingCopy.isCategoryWithOnlyProjectOperations(operationItem);
		},
		
		_itemMatchesTextFilter: function(item) {
			return this._matchesTextFilter === undefined || this._matchesTextFilter[this.getActionId(item)] === true;
		}
	});
})();
