/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForRoleTable"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsTable"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	/**
	 * Table whose rows are a hierarchy of "permissions extensions": categories,
	 * operations, and child actions for a project or team. Categories are
	 * represented as "drawers" that expand or collapse on click. Operations
	 * and actions with sub-actions can be expanded/collapsed via plus/minus
	 * expando icons. Rows that represent child actions are indented to the
	 * right of their parents' rows.
	 * 
	 * Permissions for a given role are displayed and can be modified. A
	 * different role can be specified, and the permissions and available
	 * actions to change them will be updated within this widget.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForRoleTable", [dijit._Widget, com.ibm.team.process.web.ui.internal.admin.projects.PermissionsTable], { //$NON-NLS-1$
		
		/**
		 * In addition to the arguments accepted by jazz.ui.StyledTable, this
		 * widget accepts:
		 * 
		 * model		dijit.tree.ForestStoreModel that is used to display
		 * 				a hierarchy of categories, operations, and child
		 * 				actions. The model must define functions getActionId
		 * 				and getDescription, both of which accept an item and
		 * 				return the appropriate property of that item
		 * 
		 * workingCopy	Instance of PermissionsWorkingCopy that can be
		 * 				consulted to load and update permissions
		 * 
		 * roleId		The identifier of the role that will be used to fetch 
		 * 				and display permissions for each role
		 * 
		 * contextItem	The time context that will be used to fetch and display
		 * 				permissions for each role
		 * 
		 * heightChanged	A function that will be called whenever the
		 * 					vertical space that the table will occupy with
		 * 					no scrolling changes, (eg. as rows are expanded or
		 * 					collapsed)
		 * 
		 * processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
		 * 						the Permissions tab
		 * 
		 * processAreaStore		Store of artifacts related to project or team,
		 * 						used for time scopes such as timelines and
		 * 						iteration types
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			args.columns.splice(0, 0, {
				label: this.messages.PermissionsForRoleTable_1
			});
			
			args.showHover = true;
			
			this.categoryLevel = args.categoryLevel || 0;
			this.operationLevel = args.operationLevel || 1;
			
			this._currentRoleId = args.roleId;
			this._currentContextItem = args.contextItem;
			
			this._rowsByActionId = {};
			
			this._categoryRows = [];
		},
		
		postCreate: function() {
			this.inherited(arguments);
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsForRoleTable toolbar-table"); //$NON-NLS-1$
			
			this.model.getRoot(
				dojo.hitch(this, function(item) {
					this._createRowsForChildren(item, 0);
				}),
				function(error) {
					//This should only be the result of a programming error;
					//log to the console w/ static message
					console.error(this, "error loading model root: ", error); //$NON-NLS-1$
				}
			);
			
			this.connect(this.workingCopy, "permissionChanged", "_permissionChanged"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this.connect(this.model, "onChildrenChange", "_onItemChildrenChange"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_permissionChanged: function(store, item, roleId, contextItem) {
			if (roleId === this._currentRoleId && contextItem === this._currentContextItem) {
				var row = this._rowsByActionId[store.getValue(item, ProcessConstants.ExtensionDTO.ID)];
				//Update the row's cached permission item, in case it didn't
				//have one before, (the case where a permission is configured
				//for the first time)
				row._permissionItem = item;
				this.updateRowPermissions(row);
			}
		},
		
		/**
		 * Invoked when 'OK' is pressed in the dialog to customize permissions.
		 * Expand the operation that was customized.
		 */
		_permissionsCustomized: function(row) {
			if (row._expanded === false) {
				this._toggleExpanded(row, true);
			}
		},
		
		/**
		 * Will be invoked by parent type PermissionsTable when permissions
		 * need to be loaded for the current combination of role and time
		 * period.
		 */
		fetchPermissions: function(successCallback, errorCallback) {
			delete this._permissionsStore;
			this.workingCopy.loadPermissions(
				this._currentRoleId,
				this._currentContextItem,
				successCallback,
				errorCallback
			);
		},
		
		/**
		 * Will be invoked by parent type PermissionsTable when updating
		 * a row with its current permission.
		 */
		getActionId: function(row) {
			return row._actionId;
		},
		
		/**
		 * Will be invoked by parent type PermissionsTable when updating
		 * a row with its current permission.
		 */
		getRoleId: function(row) {
			return this._currentRoleId;
		},
		
		/**
		 * Will be invoked by parent type PermissionsTable when updating
		 * a row with its current permission.
		 */
		getContextItem: function(row) {
			return this._currentContextItem;
		},
		
		/**
		 * Overriden from PermissionsTable to return a cached item if
		 * available.
		 */
		getPermissionItem: function(store, actionId) {
			var row = this._rowsByActionId[actionId];
			if (row._permissionItem === undefined || row._permissionItem === null || store !== this.getPermissionsStore() || !store.isItem(row._permissionItem)) {
				row._permissionItem = store.getPermissionItem(actionId);
			}
			return row._permissionItem;
		},
		
		/**
		 * Overriden from PermissionsTable to return a cached store if
		 * available.
		 */
		getPermissionsStore: function() {
			if (this._permissionsStore === undefined) {
				this._permissionsStore = this.workingCopy.getPermissionsStore(this._currentRoleId, this._currentContextItem);
			}
			return this._permissionsStore;
		},
		
		/**
		 * Overriden from PermissionsTable to return name of matching
		 * extension item, which requires no store lookup.
		 */
		getOperationName: function(operationId) {
			var row = this._rowsByActionId[operationId];
			return this.model.getLabel(row._item);
		},
		
		/**
		 * Update the current role which will be used to fetch permissions
		 * and update the state of each role with its appropriate permission.
		 */
		setRoleId: function(roleId) {
			this._currentRoleId = roleId;
			delete this._permissionsStore;
		},
		
		/**
		 * Update the current time period which will be used to fetch permissions
		 * and update the state of each role with its appropriate permission.
		 */
		setScope: function(contextItem) {
			this._currentContextItem = contextItem;
			delete this._permissionsStore;
		},
		
		/**
		 * Expand all expandable nodes, (categories, operations, and actions
		 * with child actions).
		 */
		expandAll: function() {
			var self = this;
			var rows = dojo.query("tr", this.tbody); //$NON-NLS-1$
			
			var self = this;
			dojo.query("tr", this.tbody).forEach(function(node) { //$NON-NLS-1$
				if (node._expanded === false) {
					self._toggleExpanded(node, false);
				}
			});
			
			this.heightChanged();
		},
		
		/**
		 * Collapse all expandable nodes, (categories, operations, and actions
		 * with child actions).
		 * 
		 * @param	includeCategories	Optional boolean that should be left
		 * 								unspecified except for very specific
		 * 								circumstances. For internal testing,
		 * 								we permit categories to be collapsed
		 * 								when a query parameter is included in
		 * 								the browser URL.
		 */
		collapseAll: function(includeCategories) {
			var self = this;
			dojo.query("tr", this.tbody).forEach(function(node) { //$NON-NLS-1$
				if (node._expanded === true && (!node._isCategory || includeCategories === true)) {
					self._toggleExpanded(node, false);
				}
			});
			
			this.heightChanged();
		},
		
		/**
		 * Expand rows so that a given set of items are revealed.
		 * 
		 * @param	items	Array of store items
		 */
		expandTo: function(items) {
			dojo.forEach(items, dojo.hitch(this, function(item) {
				var row = this._rowsByActionId[this.model.getActionId(item)];
				var parentRow = row._parentRow;
				while (parentRow !== undefined) {
					if (!parentRow._expanded) {
						this._toggleExpanded(parentRow, false);
					}
					parentRow = parentRow._parentRow;
				}
			}));
			this.heightChanged();
		},
		
		_createRowsForChildren: function(item, level, parentRow) {
			var childRows = [];
			this.model.getChildren(
				item,
				dojo.hitch(this, function(childItems) {
					dojo.forEach(childItems, dojo.hitch(this, function (childItem) {
						var newRow = this._createRows(childItem, level);
						newRow._parentRow = parentRow;
						childRows.push(newRow);
					}));
				}),
				function(error) {
					//This should only be the result of a programming error;
					//log to the console w/ static message
					console.error(this, "error loading children: ", error); //$NON-NLS-1$
				}
			);
			return childRows;
		},
		
		_createRows: function(item, level) {
			var newRow = this._primCreateRow(item, level);
			newRow._childRows = this._createRowsForChildren(item, level + 1, newRow);
			return newRow;
		},
		
		_primCreateRow: function(item, level) {
			var tr = dojo.create("tr", {}, this.tbody); //$NON-NLS-1$
			dojo.addClass(tr, "permisionsTableRow"); //$NON-NLS-1$
			tr._level = level;
			tr._item = item;
			tr._actionId = this.model.getActionId(item);
			
			this._rowsByActionId[tr._actionId] = tr;
			
			tr._nameTd = dojo.create("td", [], tr); //$NON-NLS-1$
			//labelNode is the node that directly contains the name of the
			//operation/action/category. By default it's the td whose parent
			//is the row tr, but under some circumstances it's deeper down,
			//(eg. when there is an expando node)
			tr._nameTd.labelNode = tr._nameTd;
			tr._nameTd.appendChild(document.createTextNode(this.model.getLabel(item)));
			
			//Node that can be clicked to expand/collapse
			var expandNode;
			
			if (level === this.categoryLevel) {
				//Category row
				tr._nameTd.colSpan = this.columns.length;
				dojo.addClass(tr, "categoryRow"); //$NON-NLS-1$
				
				tr._expanded = true;
				tr._isCategory = true;
				
				//Categories cannot be expanded/collapsed by default, unless a
				//query parameter permissionsDebugCollapsableCategories is
				//specified. This is used in automated tests, where we collapse
				//categories so that elements used by the test are in the
				//visible area of the screen.
				if (document.URL.indexOf("permissionsDebugCollapsableCategories") !== -1) { //$NON-NLS-1$
					expandNode = tr;
				}
				
				this._categoryRows.push(tr);
			} else {
				//Operation or sub-action row
				this.addPermissionsColumns(tr);
				
				var additionalPadding = 0;
				
				if (this.model.mayHaveChildren(item)) {
					tr._expanded = this.operationLevel === 0 && level === 0;
					
					var labelText = tr._nameTd.innerHTML;
					dojo.empty(tr._nameTd);
					
					//Create an inner table for layout, because we want the
					//name text to wrap such that each line is left-justified
					//to the right of the expand icon
					
					var innerTable = dojo.create("table", {}, tr._nameTd); //$NON-NLS-1$
					dojo.addClass(innerTable, "expandableRowContents"); //$NON-NLS-1$
					var innerTableTr = dojo.create("tr", {}, innerTable); //$NON-NLS-1$
					var innerTableTd = dojo.create("td", {}, innerTableTr); //$NON-NLS-1$
					
			    	var expandoLink = dojo.create("a", { //$NON-NLS-1$
						"href": "#" //$NON-NLS-1$ //$NON-NLS-2$
					}, innerTableTd);
					dojo.addClass(expandoLink, "expandoLink"); //$NON-NLS-1$
					
					tr._nameTd._expandoImg = dojo.create("img", {//$NON-NLS-1$
						'class': "expandoImage", //$NON-NLS-1$ //$NON-NLS-2$
						src: tr._expanded
							? dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/collapse.gif") //$NON-NLS-1$ //$NON-NLS-2$
							: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/expand.gif") //$NON-NLS-1$ //$NON-NLS-2$
					}, expandoLink);
					
					expandNode = expandoLink;
					
					this.selectRowOnFocus(tr, expandNode);
					
					//Move label text to a new node within the inner table,
					//and update labelNode since the label text is now on a
					//different node 
					innerTableTd = dojo.create("td", {}, innerTableTr); //$NON-NLS-1$
					innerTableTd.innerHTML = labelText;
					tr._nameTd.labelNode = innerTableTd;
					
					if (level !== this.operationLevel) {
						additionalPadding -= 0;
					}
				} else {
					additionalPadding = 22;
				}
				
				dojo.style(tr._nameTd, (!this.isLeftToRight() ? "paddingRight" : "paddingLeft"), (((level-this.operationLevel) * 18) + additionalPadding) + "px"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
			
			if (level === this.operationLevel) {
				this.addOperationDecorations(tr, tr._nameTd.labelNode);
			}
			
			var description = this.model.getDescription(item);
			if (null !== description && undefined !== description) {
				description = dojo.string.trim(description);
				if (0 !== description.length) {
		    		var descriptionIcon = dojo.create("a", { //$NON-NLS-1$
		    			title: description,
		    			href: "#" //$NON-NLS-1$
		    		}, tr._nameTd.labelNode);
		    		dojo.addClass(descriptionIcon, "descriptionIcon decorationIcon"); //$NON-NLS-1$
		    		
		    		var img = dojo.create("img", { //$NON-NLS-1$
		    			src: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/info_ovr.gif"), //$NON-NLS-1$ //$NON-NLS-2$
		    			title: description
		    		}, descriptionIcon);
		    		dojo.addClass(img, "rowIcon"); //$NON-NLS-1$
		    		
		    		this.connect(descriptionIcon, "onclick", function(event) { //$NON-NLS-1$
		    			dojo.stopEvent(event);
		    		});
		    		
		    		this.selectRowOnFocus(tr, descriptionIcon);
		    	}
	    	}
	    	
	    	if (expandNode) {
				var getParentRow = function(node) {
					while (node && !dojo.hasClass(node, "permisionsTableRow")) { //$NON-NLS-1$
						node = node.parentNode;
					}
					return node;
				};
				
				this.connect(expandNode, "onclick", dojo.hitch(this, function(event) { //$NON-NLS-1$
					dojo.stopEvent(event);
					this._toggleExpanded(getParentRow(event.target));
				}));
			}
			
			//By default, categories are expanded and operations are collapsed
			if (level > 1) {
				dojo.toggleClass(tr, "hidden", true); //$NON-NLS-1$
			}
			
			return tr;
		},
		
		_toggleExpanded: function(row, notifyHeightChanged) {
			row._expanded = !row._expanded;
			
    		if (row._nameTd._expandoImg) {
    			var imgSrc;
	    		if (row._expanded) {
	    			imgSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/collapse.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	    		} else {
	    			imgSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/expand.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	    		}
    			row._nameTd._expandoImg.src = imgSrc;
    		}
    		
    		if (row._childRows) {
    			var self = this;
    			var handleRow = function(nextRow) {
    				if (nextRow._childRows) {
	    				dojo.forEach(nextRow._childRows, function(childRow) {
		    				dojo.toggleClass(childRow, "hidden", !row._expanded); //$NON-NLS-1$
		    				
		    				if (!row._expanded) {
		    					self.toggleEditOperationRow(childRow, false, notifyHeightChanged);
		    				}
		    				
		    				if (!row._expanded || childRow._expanded) {
		    					handleRow(childRow);
		    				}
		    			});
		    		}
    			};
    			handleRow(row);
    		}
    		
    		if (!row._expanded) {
				self.toggleEditOperationRow(row, false, notifyHeightChanged);
			}
    		
    		if (false !== notifyHeightChanged) {
    			this.heightChanged();
    		}
		},
		
		_onItemChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList) {
			var childRows;
			if (parent === this.model.root) {
				childRows = this._categoryRows;
			} else {
				var parentRow = this._rowsByActionId[this.model.getActionId(parent)];
				childRows = parentRow._childRows;
			}
			
			if (childRows) {
				var newChildrenMap = {};
				var self = this;
				dojo.forEach(newChildrenList, function(newChild) {
					newChildrenMap[self.model.getActionId(newChild)] = true;
				});
				dojo.forEach(childRows, function(childRow) {
					self._filterOut(childRow, newChildrenMap[childRow._actionId] === undefined);
				});
				this.heightChanged();
			}
		},
		
		_filterOut: function(row, filterOut) {
			var rows = [row];
			while (rows.length > 0) {
				var row = rows.shift();
				dojo.toggleClass(row, "filteredOut", filterOut); //$NON-NLS-1$
				if (filterOut) {
					this.rowHidden(row);
				}
				if (row._childRows) {
					rows = rows.concat(row._childRows);
				}
			}
		},
		
		uninitialize: function() {
			this.inherited(arguments);
		}
	});
})();
