/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsRolesModel"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	/**
	 * Subclass of dijit.tree.ForestStoreModel that provides roles for
	 * permissions configuration to views.
	 * 
	 * Roles may optionally be filtered from view or decorated as configured.
	 * See functions setFilterConfigured() and isConfigured() respectively.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsRolesModel", [dijit.tree.ForestStoreModel], { //$NON-NLS-1$
		
		/**
		 * The following arguments are required:
		 * 
		 * store					dojo.data store of roles for the project
		 * workingCopy				Shared instance of PermissionsWorkingCopy
		 * timePeriod				Item in processAreaStore that represents
		 * 							the time period for which permissions
		 * 							are displayed. The time period determines
		 * 							whether roles are filtered out or are
		 * 							configured.
		 */
		constructor: function(args) {
			this._storeConnects = [];
			
			//Refresh the model when roles are changed
			this._storeConnects.push(dojo.connect(args.store, "onSet", this, this._refreshModel)); //$NON-NLS-1$
			
			//When permissions are changed, update filtering/highlighting and
			//refresh the model if those things might have changed. When a batch
			//of updates are made, the refresh() is deferred to the end
			
			this._storeConnects.push(dojo.connect(args.workingCopy, "startBatchUpdate", this, function() { //$NON-NLS-1$ 
				this._workingCopyBatchUpdating = true;
				delete this._refreshAfterBatch;
			}));
			this._storeConnects.push(dojo.connect(args.workingCopy, "permissionChanged", this, function(store, item, roleId, contextItem) { //$NON-NLS-1$ 
				if (contextItem === this._currentTimePeriod) {
					if (this._workingCopyBatchUpdating) {
						this._refreshAfterBatch = true;
					} else {
						this.refresh();
					}
				}
			}));
			this._storeConnects.push(dojo.connect(args.workingCopy, "endBatchUpdate", this, function() { //$NON-NLS-1$ 
				delete this._workingCopyBatchUpdating;
				if (this._refreshAfterBatch === true) {
					this.refresh();
				}
				delete this._refreshAfterBatch;
			}));
			
			this._workingCopy = args.workingCopy;
			delete args.workingCopy;
			
			this._currentTimePeriod = args.timePeriod;
			delete args.timePeriod;
			
			this._configuredRoleItems = [];
			this._configuredRoleIds = {};
			
			this._filterConfigured = false;
		},
		
		mayHaveChildren: function(item) {
			return item === this.root;
		},
		
		getLabel: function(/* item */ item) {
			return	(item === this.root) ? this.inherited(arguments) : this.store.getRoleLabel(item);
		},
		
		getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError) {
			if (parentItem === this.root) {
				this.root.children = this._filterConfigured && this._currentTimePeriod !== null
					? this._configuredRoleItems
					: this.store.getAllRoleItems();
				callback(this.root.children);
			} else {
				callback([]);
			}
		},
		
		/**
		 * Returns whether permissions are configured for the role at the
		 * current time period, (scope item that's passed in constructor and
		 * updated via setTimePeriod()).
		 */
		isConfigured: function(roleItem) {
			return roleItem !== this.root && this._configuredRoleIds[this.store.getValue(roleItem, ProcessConstants.ProcessRoleDTO.ID)];
		},
		
		/**
		 * Should be called when permissions for a different time period are
		 * to be shown. The time period determines whether roles are filtered
		 * out and impacts the result of isConfigured(). If the filtering
		 * changes, onChildrenChange() will be called on appropriate parent
		 * items, which clients can listen to.
		 * 
		 * A boolean is returned that indicates whether the filtering of items
		 * was changed as a result of the scope changing.
		 */
		setTimePeriod: function(scopeItem) {
			if (this._currentTimePeriod !== scopeItem) {
				this._currentTimePeriod = scopeItem;
				this.refresh();
				return true;
			}
			return false;
		},
		
		/**
		 * Enable/disable filtering to only show roles for which permissions
		 * are configured. If the set of roles changes, onChildrenChange() will
		 * be called on appropriate parent items, which clients can listen to.
		 */
		setFilterConfigured: function(filterConfigured) {
			if(this._filterConfigured !== filterConfigured) {
				this._filterConfigured = filterConfigured;
				this.refresh();
			}
		},
		
		/**
		 * Re-computes visible roles and result of isConfigured() and then
		 * invokes onChildrenChange() which views may listen to to update.
		 */
		refresh: function() {
			this._updateFilter();
			this._refreshModel();
		},
		
		//The following functions are overridden/customized since the
		//ForestStoreModel implementations update the root children with
		//results of a query for top level items, which does not respect
		//filtering managed by this type
		
		onNewItem: function() {
			this._refreshModel();
		},
		
		onDeleteItem: function() {
			this._refreshModel();
		},
		
		onSetItem: function() {
			setTimeout(dojo.hitch(this, this._refreshModel));
			this.inherited(arguments);
		},
		
		/**
		 * Updates fields _configuredRoleItems and _configuredRoleIds based
		 * on the current configuration of permissions and selected time period.
		 */
		_updateFilter: function() {
			this._configuredRoleIds = {};
			this._configuredRoleItems = [];
			
			var configuredRoles = this._currentTimePeriod === null
				? []
				: this._workingCopy.getConfiguredRoles(this._currentTimePeriod);
			
			
			for (var roleId in configuredRoles) {
				this._configuredRoleIds[roleId] = true;
				var query = {};
				query[ProcessConstants.ProcessRoleDTO.ID] = roleId;
				this.store.fetch({
					query: query,
					onItem: dojo.hitch(this, function(item) {
						this._configuredRoleItems.push(item);
					})
				});
			}
			
			this._configuredRoleItems.sort(dojo.hitch(this, function(a, b) {
				return this.store.getOrder(a) - this.store.getOrder(b);
			}));
		},
		
		/**
		 * Invokes onChildrenChange() on every item in the model, causing the
		 * view to completely refresh its contents.
		 */
		_refreshModel: function() {
			var items = [this.root];
			while (items.length > 0) {
				var item = items.shift();
				this.getChildren(item, dojo.hitch(this, function(newChildren) {
					this.onChildrenChange(item, newChildren);
					items = items.concat(newChildren);
				}));
			}
		},
		
		uninitialize: function() {
			dojo.forEach(this._storeConnects, dojo.disconnect);
			delete this._storeConnects;
		}
	});
})();
