/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore"); //$NON-NLS-1$

	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	/** @lends com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_PARENT_ID */
	var PROP_PARENT_ID = "parentId"; //$NON-NLS-1$
	
	/** @lends com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_PERMISSION */
	var PROP_PERMISSION = "permission"; //$NON-NLS-1$
	
	/** @lends com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_DENIAL_EXPLANATION_CONFIGURED */
	var PROP_DENIAL_EXPLANATION_CONFIGURED = "denialExplanationConfigured"; //$NON-NLS-1$
	
	/**
	 * Enumeration that is returned by functions such as getPermission()
	 * which take a combination of role, time period, and category/operation/
	 * action ID and return whether permission is granted, not granted,
	 * inherited from a parent, etc...
	 */
	var PermittedType = /** @lends com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType */ {
		PERMITTED: "PERMITTED", //$NON-NLS-1$
		
		NOT_PERMITTED: "NOT_PERMITTED", //$NON-NLS-1$
		
		/**
		 * Applies to operations, or parent actions, when all
		 * child actions are either permitted or not permitted
		 */
		SOME_PERMITTED: "SOME_PERMITTED", //$NON-NLS-1$
		
		/**
		 * Applies in a child area (aka team or project that consumes
		 * process from a provider) or when configuring for a timeline 
		 * iteration or iteration type
		 */
		INHERITED: "INHERITED" //$NON-NLS-1$
	};
	
	/**
	 * Subclass of ItemFileWriteStore for permissions that adds helper
	 * functions to retrieve and set properties of permissions.
	 * 
	 * A PermissionsStore tracks permissions for one combination of role
	 * and time period for a project or team. When editing a project
	 * or team area, there could be many distinct instances of permissions
	 * stores in the system at once.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore", dojo.data.ItemFileWriteStore, { //$NON-NLS-1$
		
		/**
		 * @param	args	Object that may contain the following:
		 * 
		 * - permissionExtensionsStore	[required] Store of permission
		 * 								extensions, typically loaded via
		 * 								PermissionsWorkingCopy::loadPermissionExtensions()
		 * 
		 * - isChildArea				[required] Whether the project/team
		 * 								for which permissions are to be tracked
		 * 								is a child area, (eg. team or process
		 * 								consumer)
		 * 
		 * - processAreaStore			[required] store with timeline/iteration
		 *								/iteration type items, (aka permissions
		 * 								scopes) for the editor
		 * 
		 * - pauseNotifications			[optional] Callback function that is
		 * 								invoked before and after a batch of
		 * 								updates is made in the store. Clients
		 * 								can use these invocations to refrain
		 * 								from responding to updates until the
		 * 								function is invoked a second time. The
		 * 								first argument is a reference to the
		 * 								permissions store. The second is a
		 * 								boolean 'true' before the batch is
		 * 								started and 'false' afterward.
		 */
		constructor: function(args) {
			this.data = {
				identifier: 'id', //$NON-NLS-1$
				label: 'name', //$NON-NLS-1$
				items: []
			};
			
			this._permissionExtensionsStore = args.permissionExtensionsStore;
			this._isChildArea = args.isChildArea;
			this._processAreaStore = args.processAreaStore;
			
			this._pauseNotifications = args.pauseNotifications || function() {};
		},
		
		/*
		 * Create entries in the store for one operation and all of its child actions
		 * by copying the structure of the extensions store. If permission is not null 
		 * every every item created in the store will have the permission attribute set 
		 * to that value
		 */
		_createStoreEntry: function(operationId, permission) {
			//Pause notifications while the item is being created. Listeners
			//will be notified when/if the item is actually modified after this
			//function is called
			this._pauseNotifications(this, true);
			
			this._permissionExtensionsStore.fetch({
				query: {
					id: "*" + operationId //$NON-NLS-1$
				},
				queryOptions: {
					deep: true
				},
				onItem: dojo.hitch(this, function (item) {
					this._copyFromExtensionStore(item, permission);
				})
			});
			
			this._pauseNotifications(this, false);
		},
		
		/*
		 * Queries the store to see if an entry for an operationId exists, if not
		 * one is created. If permission is not null every item created in the store
		 * will get that permission value
		 */
		ensureOperationInStore: function(operationId, permission) {
			this.fetch({
				query: {
					id: "*" + operationId //$NON-NLS-1$
				},
				queryOptions: {
					deep: true
				},
				onComplete: dojo.hitch(this, function (items) {
					if (items.length == 0) {
						this._createStoreEntry(operationId, permission);
					}
				})
			});
		},
		
		/*
		 * Queries the store to see if an entry for the parent operation exists, if not
		 * one is created. If permission is not null every item created in the store
		 * will get that permission value
		 */
		_ensureActionInStore: function(actionId, permission) {
		    this.ensureOperationInStore(this._getParentOperationId(actionId), permission);
		},
		
		/*
		 * Deep copy an element from the extensions store to the permissions store as a top level item
		 * returns the item created. If permission is not null every item created in the store will have 
		 * the permission attribute set to that value
		 */
		_copyFromExtensionStore: function(sourceItem, permission) {
			var targetItem = this.newItem({id: this._permissionExtensionsStore.getValue(sourceItem, ProcessConstants.ExtensionDTO.ID)});
			this._copyAttributesFromExtensionStore(sourceItem, targetItem, permission);
			this._copyChildrenFromExtensionStore(sourceItem, targetItem, permission);
			return targetItem;
		},
		
		/*
		 * Deep copy an elements children from the extensions store to the permissions store
		 * If permission is not null every item created in the store will have the permission 
		 * attribute set to that value
		 */
		_copyChildrenFromExtensionStore: function(sourceParent, targetParent, permission) {			
			var children = this._permissionExtensionsStore.getValues(sourceParent, ProcessConstants.ExtensionDTO.ITEMS);
			for (var childIndex = 0; childIndex < children.length; childIndex++) {
				var sourceChild = children[childIndex];
				var targetChild = this.newItem({
					id: this._permissionExtensionsStore.getValue(sourceChild, ProcessConstants.ExtensionDTO.ID)}, 
					{parent: targetParent, attribute: "items"}); //$NON-NLS-1$
				this._copyAttributesFromExtensionStore(sourceChild, targetChild, permission);
				this._copyChildrenFromExtensionStore(sourceChild, targetChild, permission);
	        }
		},
		
		_attributesOfInterest: [
			ProcessConstants.PermissionConfigurationDTO.FINAL_OP,
			ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION,
			ProcessConstants.ExtensionDTO.CONTEXT,
			PROP_PARENT_ID
		],
		
		/*
		 * Copy the attributes from an item in the extension store to an item in
		 * the permissions store. To save on space in the permissions store only 
		 * copy the attributes that are needed.
		 */
		_copyAttributesFromExtensionStore: function(sourceItem, targetItem, permission) {
			for(var index = 0; index < this._attributesOfInterest.length; index++){
				var attributeName = this._attributesOfInterest[index];
				var value = this._permissionExtensionsStore.getValue(sourceItem, attributeName);
				if (value !== undefined) {
					this.setValue(targetItem, attributeName, value);
				}
			}
			if (permission) {
				this.setValue(targetItem, PROP_PERMISSION, permission);
			}
		},
		
		_getParentOperationId: function(actionId) {
			var segments = actionId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			if (segments.length <= 2) {
				return actionId;
			} 
			return segments[0] + ProcessConstants.PERMISSION_PATH_SEPARATOR + segments[1];
		},
		
		/**
		 * Return a dojo.data store item for a store and operation/action
		 * identifier. The result may be null if permissions have not been
		 * configured for the combination of store (itself a combination of
		 * role and time period) and operation/action identifier.
		 */
		getPermissionItem: function(actionId) {
			var item = null;
			this.fetch({
				query: {
					id: actionId 
				},
				queryOptions: {
					deep: true
				},
				onComplete: function(items) {
					if (items.length > 0) {
						item = items[0];
				    }
				}
			});
			return item;
		},
		
		/**
		 * Given a permissions store and a permission item, returns whether the
		 * action is permitted, not permitted, inherited, or a combination of
		 * the above, (see the values of enumeration
		 * com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType,
		 * one of which is returned).
		 * 
		 * @param	item		Item to get permission of. See getPermissionItem().
		 * @param	contextItem	The context, (time period) to get permission for.
		 * 						Useful to return default value if permission is
		 * 						not configured.
		 * 
		 * @return com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType
		 */
		getPermission: function(item, contextItem) {
			if (item === null) {
				return this._isChildArea || contextItem !== this._processAreaStore.alwaysRootItem ? PermittedType.INHERITED : PermittedType.NOT_PERMITTED;
			} else {
				return this.getValue(item, PROP_PERMISSION);
			}
		},
		
		/**
		 * Returns whether the operation is final.
		 * 
		 * @param	item		Item to get permission of. See getPermissionItem().
		 * 
		 * @return true if the operation if final, false otherwise
		 */
		isFinal: function(item) {
			if (item === null) {
				return false;
			}
			var finalOp = this.getValue(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP);
			if (finalOp === undefined) {
			 	finalOp = false;
			}
			return finalOp;
		},
		
		/**
		 * Returns the explanation to be shown if permission for that operation
		 * is denied.
		 * 
		 * @param	item		Item to get permission of. See getPermissionItem().
		 * 
		 * @return the denial explanation or null if not configured
		 */
		getDenialExplanation: function(item) {
			if (item === null) {
				return null;
			}
			var denialExplanation = this.getValue(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION);
			if (denialExplanation === undefined) {
				denialExplanation = null; //$NON-NLS-1$
			}
			return denialExplanation;
		},
		
		/**
		 * Returns a boolean which indicates whether a denial explanation was
		 * configured.
		 * 
		 * @param	item		Item to get permission of. See getPermissionItem().
		 * 
		 * @return true if a denial explanation was configured, otherwise false
		 */
		isDenialExplanationConfigured: function(item) {
			if (item === null) {
				return false;
			}
			var explanationConfigured = this.getValue(item, PROP_DENIAL_EXPLANATION_CONFIGURED);
			var explanation = this.getValue(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION);
			if (explanation === undefined) {
				explanation = null;
			}
			return false !== explanationConfigured && (true === explanationConfigured || explanation !== null);
		},
		
		/**
		 * Sets the permission for a given role, time period, and the identifier of a category, operation,
		 * or action. The permission state of child actions and ancestors will also be set as necessary
		 * based on the new permission value.
		 * 
		 * @param	item		Item to get permission of. See getPermissionItem().
		 * @param	actionId	Identifier of operation or action to set permission
		 * 						for. Used if item is null to create item in store
		 * @param   permission  New value of the permission, may be PERMITTED, NOT_PERMITTED or INHERITED
		 * @param	contextItem	The context, (time period) to set permission for.
		 * 						Useful to return default value if permission is
		 * 						not configured.
		 */
		setPermission: function(item, actionId, contextItem, permission) {
			var unconfiguredValue = this._isChildArea || contextItem !== this._processAreaStore.alwaysRootItem ? PermittedType.INHERITED : PermittedType.NOT_PERMITTED;
			if (item === null) {		
				this._ensureActionInStore(actionId, unconfiguredValue);
				this.fetchItemByIdentity({
					identity: actionId,
					onItem: function(theItem) {
						item = theItem;
					}
				});
			}
				
			//If setting to inherited, clear the 'is final' and 'denial
			//text' properties of the parent operation
			if (PermittedType.INHERITED === permission) {
				this._pauseNotifications(this, true);
				if (this.isFinal(item)) {
					this.setFinal(item, actionId, false);
				}
				if (this.isDenialExplanationConfigured(item)) {
					this.setDenialExplanationConfigured(item, actionId, false);
				}
				this._pauseNotifications(this, false);
			}
			var self = this;
			
			var permitItemAndChildren = function(item) {
				self.setValue(item, PROP_PERMISSION, permission);
				
				dojo.forEach(self.getValues(item, ProcessConstants.ExtensionDTO.ITEMS), function(childItem) {
					permitItemAndChildren(childItem);
				});
			};
			
			permitItemAndChildren(item);
			
			var calculateAndSetParentPermission = function(item) {
				var allOperationsPermitted = true;
				var noOperationsPermitted = true;
				var allOperationsInherited = true;
				var someOperationsInherited = false;
				
				dojo.forEach(self.getValues(item, ProcessConstants.ExtensionDTO.ITEMS), function(childItem) {
					var childPermission = self.getValue(childItem, PROP_PERMISSION);
					if (PermittedType.PERMITTED === childPermission) {
						noOperationsPermitted = false;
						allOperationsInherited = false;
					} else if (PermittedType.NOT_PERMITTED === childPermission) {
						allOperationsPermitted = false;
						allOperationsInherited = false;
					} else if (PermittedType.SOME_PERMITTED === childPermission) {
						allOperationsPermitted = false;
						noOperationsPermitted = false;
						allOperationsInherited = false;
					} else if (PermittedType.INHERITED === childPermission) {
						allOperationsPermitted = false;
						someOperationsInherited = true;
					}
				});
					
				if (allOperationsPermitted) {
					self.setValue(item, PROP_PERMISSION, PermittedType.PERMITTED);
				} else if (noOperationsPermitted) {
					self.setValue(item, PROP_PERMISSION, PermittedType.NOT_PERMITTED);
				} else if (allOperationsInherited) {
					self.setValue(item, PROP_PERMISSION, PermittedType.INHERITED);
				} else {
					self.setValue(item, PROP_PERMISSION, PermittedType.SOME_PERMITTED);
				}
			};
			
			var parentId = this.getValue(item, PROP_PARENT_ID);
			while (parentId !== undefined) {
				this.fetchItemByIdentity({
					identity: parentId,
					onItem: function(parentItem) {
						if (parentItem === null) {
							parentId = undefined;
						} else {
							calculateAndSetParentPermission(parentItem);
							parentId = self.getValue(parentItem, PROP_PARENT_ID);
						}
					}
				});
			}
		},
		
		/**
		 * Sets the isFinal flag for an operation
		 * 
		 * @param	item		Item to set property of. See getPermissionItem().
		 * @param	operationId	Identifier of operation to configure. Used if
		 * 						the item is null to create an item in the store 
		 * @param   isFinal     New value of isFinal as boolean
		 */
		setFinal: function(item, operationId, isFinal) {
			// Ensure that the operation is loaded in the permissions store. If the
			// store item needs to be created set its permission to NOT_PERMITTED
			if (item === null) {
				item = this.ensureOperationInStore(operationId, PermittedType.NOT_PERMITTED);
				this.fetchItemByIdentity({
					identity: operationId,
					onItem: function(theItem) {
						item = theItem;
					}
				});
			}
			
			this.setValue(item, ProcessConstants.PermissionConfigurationDTO.FINAL_OP, isFinal);
		},
		
		/**
		 * Sets the denial explanation for an operation
		 *
		 * @param	item		Item to set property of. See getPermissionItem().
		 * @param	operationId	Identifier of operation to configure. Used if
		 * 						the item is null to create an item in the store
		 * @param   denialText  New value of the denial text
		 */
		setDenialExplanation: function(item, operationId, denialText) {
			// Ensure that the operation is loaded in the permissions store. If the
			// store item needs to be created set its permission to NOT_PERMITTED
			if (item === null) {
				item = this.ensureOperationInStore(operationId, PermittedType.NOT_PERMITTED);
				this.fetchItemByIdentity({
					identity: operationId,
					onItem: function(theItem) {
						item = theItem;
					}
				});
			}
			
			this.setValue(item, ProcessConstants.PermissionConfigurationDTO.DENIAL_EXPLANATION, denialText);
		},
		
		/**
		 * Sets whether the denial explanation is configured for an operation
		 * 
		 * @param	item		Item to set property of. See getPermissionItem().
		 * @param	operationId	Identifier of operation to configure. Used if
		 * 						the item is null to create an item in the store
		 * @param   configured  boolean with a value of true if the denial text is configured, false otherwise
		 * 
		 */
		setDenialExplanationConfigured: function(item, operationId, configured) {
			// Ensure that the operation is loaded in the permissions store. If the
			// store item needs to be created set its permission to NOT_PERMITTED
			if (item === null) {
				item = this.ensureOperationInStore(operationId, PermittedType.NOT_PERMITTED);
				this.fetchItemByIdentity({
					identity: operationId,
					onItem: function(theItem) {
						item = theItem;
					}
				});
			}
			
			this.setValue(item, PROP_DENIAL_EXPLANATION_CONFIGURED, configured);
		}
	});
	
	com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType = PermittedType;
	com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_PARENT_ID = PROP_PARENT_ID;
	com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_PERMISSION = PROP_PERMISSION;
	com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PROP_DENIAL_EXPLANATION_CONFIGURED = PROP_DENIAL_EXPLANATION_CONFIGURED;
})();
