/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.viewHistory");
dojo.require("net.jazz.ajax.hasLocalRepository");
dojo.require("net.jazz.ajax.dataServerRoot");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("com.ibm.team.repository.web.client.session.Session");

/**
 * Manages view history for application pages. Pages are expected
 * to log newly viewed resources and ask for history as they are building
 * the 'Recently Viewed' group of their component menus. View history
 * is persisted using HTML5 web history for browsers that support it,
 * or in session only for those that don't. If local repository is
 * detected, view history service will be used instead and items will be
 * persisted in the repository.
 */
(function(){
    var entries = {};
    var hasLocalRepository = net.jazz.ajax.hasLocalRepository();
    var contextRoot = net.jazz.ajax.dataServerRoot() || net.jazz.ajax.contextRoot();
    var KEY_PREFIX = "jazz.app.viewHistory";
    var ROOT = "%ROOT%";
    var MISSING = "--";
    var MY_APPLICATION = "%MY_APPLICATION%";
    var APP = jazz.app.internal.applicationAbout.findAboutUriFromContext(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL);
    
    jazz.app.viewHistory = {
        /**
         * Returns a deferred that is passed an array of objects containing all the required information
         * to build menu items for the 'Recently Viewed' group of the component menu.
         *
         * @param {Object} query an object containing scope information required to
         * return the correct history. Accepted query properties are:
         *    - 'application' - a value normally specified as application:about. If the
         *    value passed is "%MY_APPLICATION%", it will be replaced with the computed
         *    value as defined by the 'about' service and provided by BootstrapProperties.
         *    - 'project' - name of the current project area
         *    - 'type' - type of the entry (e.g. 'com.ibm.team.dashboard').
         *    - 'size' - number of items to return (if not set, default is 5)
         *    - 'startIndex' - where to start (if not set, default is 0)
         *    - 'exclude' - an Array of URIs that will be used to exclude the matching items. This
         *    feature should be used to exclude unsaved items from the list.
         * @param {boolean} nocache if set to true, it will force a fresh call and
         * invalidate the cache; otherwise, once the first call is made, subsequent
         * calls will return the state from the cache (default is false).
         * @return {dojo.Deferred} a dojo.Deferred object indicating success or failure
         *  of the history get attempt. If successful, functions registered for callback
         *  will be passed an object with two properties: 'count' that represents
         *  total number of available items, and 'items' that is an Array of actual
         *  result objects. Note that when 'size' and/or 'startIndex' properties
         *  are used in the query, 'count' may be greater than the length of the
         *  'items' array.
         */
        get: function(query, nocache){
            //TODO rework 'get' for local storage to support partial
            //queries (this only affects use cases other than component menus)
            query.user = com.ibm.team.repository.web.client.session.getAuthenticatedContributor().userId;
            if (query.application == MY_APPLICATION) 
                query.application = APP;
            var key = this._createKey(query);
            //var history = this._getMergedCache(key, query);
			var history = entries[key];
            
            if (!nocache && history) {
                // bypass loading - use cached but respect exclusions
                return this._getCached(history, query);
            }
            if (hasLocalRepository) 
                return this._getService(key, query);
            else {
                if (typeof(localStorage) != "undefined") {
                    this._initFromLocalStorage(key);
                }
                return this._getCached(history, query);
            }
        },
        
        /**
         * @param {Object} query the object equivalent to the object used to
         * query this item. It contains data common to the resource type: 'application', 'project'
         * and 'type'. Since it is common it can be reused between invocations.
         * Note that if 'application' is set to "%MY_APPLICATION%, the actual
         * value will be computed by calling BoostrapProperties, which is
         * equivalent to what application:about service provides.
         * @param {Object} item a new item to put in history as the most recently
         * viewed. It should contain properties 'label', 'uri' and optionally 'iconUri'. 
         * If relative urls are provided for 'uri' or 'iconUri', they will be made absolute
         * before posting to the server using BootstrapProperties.FRONTSIDE_URL.
         * In addition, it can contain a property 'data' that consumers can use to
         * store opaque data field for subsequent usage. This field is not
         * a queriable property.
         */
        put: function(query, item){
            if (query.application == MY_APPLICATION) 
                query.application = APP;
            if (!query.project) 
                query.project = ROOT;
            var entry = {
                application: query.application,
                project: query.project,
                type: query.type,
                label: item.label,
                uri: this._fullyQualify(item.uri),
                iconUri: this._fullyQualify(item.iconUri),
                data: item.data
            };
            query.user = com.ibm.team.repository.web.client.session.getAuthenticatedContributor().userId;
            var key = this._createKey(query);
            
            if (!hasLocalRepository && typeof(localStorage) != "undefined") {
                // put may be called before get - initialize
                this._initFromLocalStorage(key);
            }
            var history = entries[key];
            
            if (history) { //if local repository is true, history can be undefined
                // Look for the entry with the same uri
                for (var i = 0; i < history.length; i++) {
                    if (history[i].uri == entry.uri) {
                        // duplicate - remove
                        history.splice(i, 1);
                        break;
                    }
                }
                //add it as first
                history.unshift(entry);
                // trim the size (we are keeping 10 entries)
                var extra = history.length - 10;
                if (extra > 0) 
                    history.splice(10, extra);
            }
            
            if (hasLocalRepository) 
                this._postService(entry);
            else {
                if (typeof(localStorage) != "undefined") {
                    localStorage.setItem(key, dojo.toJson(history));
                }
            }
        },
		
		/*
		 * If the URL starts with the context root, make it absolute
		 */
		_fullyQualify: function(url) {
			if (!url) return url;
			var cr = net.jazz.ajax.contextRoot();
			if (url.indexOf(cr) === 0) {
				url = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + url.substring(cr.length);
			}
			return url;
		},
        
		/*
		 * Merge content of multiple caches when certain properties (project and/or type)
		 * in the query are missing.
		 */
        _getMergedCache: function(key, query){
            //shortcut - if project and type is defined, we can 
            //use a single cache array

            if (query.project && query.type) 
                return entries[key];
            //We need to loop
            var result;
            
            for (var ckey in entries) {
                var keyArray = ckey.split("|");
				console.debug(ckey);
				console.debug(query);

                if (query.application && query.application != keyArray[1]) 
                    continue;
                if (query.project && query.project != keyArray[2]) 
                    continue;
                if (query.type && query.type != keyArray[3])
                    continue;
                if (query.user && query.user != keyArray[4]) 
                    continue;
                // no explicit property or full match - merge
                if (!result)
                    result = entries[ckey];
                else
                    result = result.concat(entries[ckey]);
				console.debug(result);
            }
            /*
             result.sort(function(a,b) {
             });
             */
            return result;
        },
        
        /*
         * User xhr GET to call the local history service
         */
        _getService: function(key, query){
            var xhrArgs = {
                url: this._assembleURL(query),
                // ask the server to send JSON and expect form encoded data
                headers: {
                    "accept": "application/json"
                },
                handleAs: "json"
            };
            var deferred = new dojo.Deferred();
            // call the view history service using the GET method
            jazz.client.xhrGet(xhrArgs).addCallbacks(function(result){
                //cache the result so that we don't have to call the service again
                entries[key] = result ? result.items : [];
                deferred.callback(result);
            }, function(error){
                deferred.errback(error);
            });
            return deferred;
        },
        
        /*
         * Use xhr POST to push the new item using the local history service.
         */
        _postService: function(entry){
            var xhrArgs = {
                url: this._assembleURL(),
                // ask the server to send JSON and expect form encoded data
                headers: {
                    "content-type": "application/x-www-form-urlencoded; charset=utf-8",
                    "X-com-ibm-team-configuration-versions": "LATEST",
                    "X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
                }
            };
            
            xhrArgs.postData = dojo.objectToQuery(entry);
            
            // call the view history service using the POST method
            var deferred = jazz.client.rawXhrPost(xhrArgs);
            deferred.addErrback(dojo.hitch(this, function(error){
                //TODO handle the post error somehow - log?
            }));
        },
        
        /*
         * Compute the URL for the view history service call, optionally
         * adding query parameters if passed.
         */
        _assembleURL: function(params){
            var url = contextRoot + "/" + "view-history";
            
            // if this is a GET request, append the query string
            if (params) 
                url += "?" + dojo.objectToQuery(params);
            return url;
        },
        
        /*
         * Returns a key to be used for storing items for this query.
         * Key is stable in its length - it contains five segments separated by '|':
         * "prefix|application|project|type|user"
         * where missing segments are replaced with '--'. Partial queries
         * are possible by iterating over the localStorate items that start
         * with our prefix and matching keys that are not missing,
         * including those that are.
         */
        _createKey: function(query){
            var key = [];
            key.push(KEY_PREFIX);
            if (query.application) 
                key.push(query.application);
            else 
                key.push(MISSING);
            if (query.project) 
                key.push(query.project);
            else 
                key.push(MISSING);
            key.push(query.type);
            if (query.user) 
                key.push(query.user);
            else 
                key.push(MISSING);
            return key.join("|");
        },
        
        /*
         * Loads the entries from local storage into the cache. If not found,
         * will not try again.
         */
        _initFromLocalStorage: function(key){
            if (!entries[key]) {
                var history = dojo.fromJson(localStorage.getItem(key));
                entries[key] = history ? history : [];
            }
        },
        
        /*
         * Returns cached items based on the query while excluding
         * items that match URIs listed in exclusions. The result
         * should contain up to 5 items.
         */
        _getCached: function(history, query){
            var items = [];
            var startIndex = query.startIndex ? query.startIndex : 0;
            var size = query.size ? query.size : 5;
            if (history) {
                for (var i = 0; i < history.length; i++) {
                    // test for exclusions
                    if (query.exclude && dojo.indexOf(query.exclude, history[i].uri) != -1) 
                        continue;
                    if (i < startIndex) 
                        continue;
                    items.push(history[i]);
                    if (items.length == size) 
                        break;
                }
            }
            var def = new dojo.Deferred();
            def.callback({
                count: history ? history.length : 0,
                items: items
            });
            return def;
        }
    };
    
})();
